# mode: run
# tag: monitoring, trace
# cython: language_level=3
# cython: profile=True, linetrace=True
# distutils: define_macros = CYTHON_TRACE_NOGIL=1

import cython

import operator
from collections import defaultdict
from contextlib import contextmanager
from functools import partial

COMPILED = cython.compiled

try:
    from sys import monitoring as smon
    E = smon.events
except ImportError:
    TOOL_ID = 0
    FUNC_EVENTS = GEN_EVENTS = ()

    __doc__ = """
    >>> True
    True
    """
else:
    TOOL_ID = smon.PROFILER_ID
    FUNC_EVENTS = (E.PY_START, E.PY_RETURN, E.RAISE)
    GEN_EVENTS = FUNC_EVENTS + (E.PY_RESUME, E.PY_YIELD)  # , E.STOP_ITERATION)
    event_name = {1 << event_id: name for name, event_id in vars(E).items()}.get

    ####### Debug helper
    # Use like:
    #   python3 -c 'import sys_monitoring as sm; sm.print_event_traces()' > pytrace.log
    # Then compile the module and run:
    #   python3 -c 'import sys_monitoring as sm; sm.print_event_traces()' > cytrace.log
    # Compare the two logs.

    ALL_EVENTS = sorted([(name, value) for (name, value) in vars(E).items() if value != 0], key=operator.itemgetter(1))

    @cython.profile(False)
    @cython.linetrace(False)
    def print_event_traces():
        trace_events(test_profile)
        trace_events(test_generators)

    @cython.profile(False)
    @cython.linetrace(False)
    def trace_events(func=None, events=ALL_EVENTS):
        event_set = E.NO_EVENTS
        for _, event in events:
            event_set |= event

        last_code_obj = [None]

        @cython.profile(False)
        @cython.linetrace(False)
        def print_event(event, code_obj, offset, *args):
            if last_code_obj[0] is not code_obj:
                last_code_obj[0] = code_obj
                print()
            print(f"{event}, {code_obj.co_name}:{code_obj.co_firstlineno}, {offset}{', ' if args else ''}", *args)

        try:
            smon.use_tool_id(TOOL_ID, 'cython-test')
            for name, event in events:
                smon.register_callback(TOOL_ID, event, partial(print_event, name))
            smon.set_events(TOOL_ID, event_set)

            result = func(1) if func is not None else test_profile(1)

        finally:
            smon.set_events(TOOL_ID, event_set)
            for _, event in events:
                smon.register_callback(TOOL_ID, event, None)
            smon.free_tool_id(TOOL_ID)

        return result

    ####### Doctests

    __doc__ = """
    >>> from itertools import combinations

    >>> smon.use_tool_id(TOOL_ID, 'cython-test')

    >>> with monitored_events(events=()) as collected_events:
    ...     print(test_profile(0))
    0
    >>> list(collected_events.items())  # test_profile(0)
    []

    >>> with monitored_events(events=()) as collected_events:
    ...     print(test_profile(1))
    0
    >>> list(collected_events.items())  # test_profile(1)
    []

    >>> with monitored_events(events=()) as collected_events:
    ...     print(test_profile(2))
    22
    >>> list(collected_events.items())  # test_profile(2)
    []


    ## Testing combinations of events:

    >>> for num_events in range(1, len(FUNC_EVENTS)+1):  # doctest: +REPORT_NDIFF
    ...     for event_set in combinations(FUNC_EVENTS, num_events):
    ...         print(f"--- {names(event_set)} ---")
    ...         with monitored_events(events=event_set + (E.RERAISE, E.PY_UNWIND)) as collected_events:
    ...             result = test_profile(10)
    ...         print(result)
    ...         print_events(collected_events)
    ...         assert_events(event_set, collected_events['test_profile'])  # test_profile(10)
    --- PY_START ---
    990
    f_cdef PY_START [10]
    f_cpdef PY_START [20]
    f_def PY_START [10]
    f_inline PY_START [10]
    f_inline_prof PY_START [10]
    f_nogil_prof PY_START [10]
    f_raise PY_START [20], PY_UNWIND [20]
    f_reraise PY_START [10], PY_UNWIND [10], RERAISE [10]
    f_return_default PY_START [10]
    f_return_none PY_START [10]
    f_withgil_prof PY_START [10]
    m_cdef PY_START [10]
    m_classmethod PY_START [10]
    m_cpdef PY_START [20]
    m_def PY_START [20]
    m_staticmethod PY_START [10]
    test_profile PY_START [1]
    --- PY_RETURN ---
    990
    f_cdef PY_RETURN [10]
    f_cpdef PY_RETURN [20]
    f_def PY_RETURN [10]
    f_inline PY_RETURN [10]
    f_inline_prof PY_RETURN [10]
    f_nogil_prof PY_RETURN [10]
    f_raise PY_UNWIND [20]
    f_reraise PY_UNWIND [10], RERAISE [10]
    f_return_default PY_RETURN [10]
    f_return_none PY_RETURN [10]
    f_withgil_prof PY_RETURN [10]
    m_cdef PY_RETURN [10]
    m_classmethod PY_RETURN [10]
    m_cpdef PY_RETURN [20]
    m_def PY_RETURN [20]
    m_staticmethod PY_RETURN [10]
    test_profile PY_RETURN [1]
    --- RAISE ---
    990
    f_raise RAISE [20], PY_UNWIND [20]
    f_reraise RAISE [10], PY_UNWIND [10], RERAISE [10]
    test_profile RAISE [20]
    --- PY_START, PY_RETURN ---
    990
    f_cdef PY_START [10], PY_RETURN [10]
    f_cpdef PY_START [20], PY_RETURN [20]
    f_def PY_START [10], PY_RETURN [10]
    f_inline PY_START [10], PY_RETURN [10]
    f_inline_prof PY_START [10], PY_RETURN [10]
    f_nogil_prof PY_START [10], PY_RETURN [10]
    f_raise PY_START [20], PY_UNWIND [20]
    f_reraise PY_START [10], PY_UNWIND [10], RERAISE [10]
    f_return_default PY_START [10], PY_RETURN [10]
    f_return_none PY_START [10], PY_RETURN [10]
    f_withgil_prof PY_START [10], PY_RETURN [10]
    m_cdef PY_START [10], PY_RETURN [10]
    m_classmethod PY_START [10], PY_RETURN [10]
    m_cpdef PY_START [20], PY_RETURN [20]
    m_def PY_START [20], PY_RETURN [20]
    m_staticmethod PY_START [10], PY_RETURN [10]
    test_profile PY_START [1], PY_RETURN [1]
    --- PY_START, RAISE ---
    990
    f_cdef PY_START [10]
    f_cpdef PY_START [20]
    f_def PY_START [10]
    f_inline PY_START [10]
    f_inline_prof PY_START [10]
    f_nogil_prof PY_START [10]
    f_raise PY_START [20], RAISE [20], PY_UNWIND [20]
    f_reraise PY_START [10], RAISE [10], PY_UNWIND [10], RERAISE [10]
    f_return_default PY_START [10]
    f_return_none PY_START [10]
    f_withgil_prof PY_START [10]
    m_cdef PY_START [10]
    m_classmethod PY_START [10]
    m_cpdef PY_START [20]
    m_def PY_START [20]
    m_staticmethod PY_START [10]
    test_profile PY_START [1], RAISE [20]
    --- PY_RETURN, RAISE ---
    990
    f_cdef PY_RETURN [10]
    f_cpdef PY_RETURN [20]
    f_def PY_RETURN [10]
    f_inline PY_RETURN [10]
    f_inline_prof PY_RETURN [10]
    f_nogil_prof PY_RETURN [10]
    f_raise RAISE [20], PY_UNWIND [20]
    f_reraise RAISE [10], PY_UNWIND [10], RERAISE [10]
    f_return_default PY_RETURN [10]
    f_return_none PY_RETURN [10]
    f_withgil_prof PY_RETURN [10]
    m_cdef PY_RETURN [10]
    m_classmethod PY_RETURN [10]
    m_cpdef PY_RETURN [20]
    m_def PY_RETURN [20]
    m_staticmethod PY_RETURN [10]
    test_profile PY_RETURN [1], RAISE [20]
    --- PY_START, PY_RETURN, RAISE ---
    990
    f_cdef PY_START [10], PY_RETURN [10]
    f_cpdef PY_START [20], PY_RETURN [20]
    f_def PY_START [10], PY_RETURN [10]
    f_inline PY_START [10], PY_RETURN [10]
    f_inline_prof PY_START [10], PY_RETURN [10]
    f_nogil_prof PY_START [10], PY_RETURN [10]
    f_raise PY_START [20], RAISE [20], PY_UNWIND [20]
    f_reraise PY_START [10], RAISE [10], PY_UNWIND [10], RERAISE [10]
    f_return_default PY_START [10], PY_RETURN [10]
    f_return_none PY_START [10], PY_RETURN [10]
    f_withgil_prof PY_START [10], PY_RETURN [10]
    m_cdef PY_START [10], PY_RETURN [10]
    m_classmethod PY_START [10], PY_RETURN [10]
    m_cpdef PY_START [20], PY_RETURN [20]
    m_def PY_START [20], PY_RETURN [20]
    m_staticmethod PY_START [10], PY_RETURN [10]
    test_profile PY_START [1], PY_RETURN [1], RAISE [20]


    ## Testing generators:

    >>> with monitored_events(events=GEN_EVENTS, function_name='f_generator') as collected_events:
    ...     gen = f_generator()
    ...     for i in gen: print(i)
    1
    2
    >>> print_events(collected_events)  # f_generator
    f_generator PY_START [1], PY_RESUME [2], PY_RETURN [1], PY_YIELD [2]


    >>> with monitored_events(events=GEN_EVENTS, function_name='test_generators') as collected_events:
    ...     test_generators()
    >>> print_events(collected_events)  # test_generators()
    f_generator PY_START [1], PY_RESUME [2], PY_RETURN [1], PY_YIELD [2]
    f_generator_exception PY_START [1], PY_RESUME [1], PY_YIELD [1], RAISE [1]
    f_generator_expr PY_START [1], PY_RETURN [1]
    test_generators PY_START [1], PY_RETURN [1]


    ## Testing line events:

    >>> events = (E.PY_START, E.PY_RETURN)
    >>> for num_events in range(1, len(events)+1):  # doctest: +REPORT_NDIFF
    ...     for event_set in combinations(events, num_events):
    ...         event_set += (E.RAISE, E.LINE)
    ...         print(f"--- {names(event_set)} ---")
    ...         with monitored_events(events=event_set + (E.RERAISE, E.PY_UNWIND)) as (collected_events, collected_line_events):
    ...             result = test_profile(10)
    ...         print(result)
    ...         print_events(collected_events)
    ...         assert_events(event_set, collected_events['test_profile'])  # test_profile(10)
    --- PY_START, LINE, RAISE ---
    990
    f_cdef PY_START [10], LINE [10]
    f_cpdef PY_START [20], LINE [20]
    f_def PY_START [10], LINE [10]
    f_inline PY_START [10], LINE [10]
    f_inline_prof PY_START [10], LINE [10]
    f_nogil_prof PY_START [10], LINE [10]
    f_raise PY_START [20], LINE [20], RAISE [20], PY_UNWIND [20]
    f_reraise PY_START [10], LINE [50], RAISE [10], PY_UNWIND [10], RERAISE [10]
    f_return_default PY_START [10], LINE [10]
    f_return_none PY_START [10], LINE [10]
    f_withgil_prof PY_START [10], LINE [10]
    m_cdef PY_START [10], LINE [10]
    m_classmethod PY_START [10], LINE [10]
    m_cpdef PY_START [20], LINE [20]
    m_def PY_START [20], LINE [20]
    m_staticmethod PY_START [10], LINE [10]
    test_profile PY_START [1], LINE [385], RAISE [20]
    --- PY_RETURN, LINE, RAISE ---
    990
    f_cdef PY_RETURN [10], LINE [10]
    f_cpdef PY_RETURN [20], LINE [20]
    f_def PY_RETURN [10], LINE [10]
    f_inline PY_RETURN [10], LINE [10]
    f_inline_prof PY_RETURN [10], LINE [10]
    f_nogil_prof PY_RETURN [10], LINE [10]
    f_raise LINE [20], RAISE [20], PY_UNWIND [20]
    f_reraise LINE [50], RAISE [10], PY_UNWIND [10], RERAISE [10]
    f_return_default PY_RETURN [10], LINE [10]
    f_return_none PY_RETURN [10], LINE [10]
    f_withgil_prof PY_RETURN [10], LINE [10]
    m_cdef PY_RETURN [10], LINE [10]
    m_classmethod PY_RETURN [10], LINE [10]
    m_cpdef PY_RETURN [20], LINE [20]
    m_def PY_RETURN [20], LINE [20]
    m_staticmethod PY_RETURN [10], LINE [10]
    test_profile PY_RETURN [1], LINE [385], RAISE [20]
    --- PY_START, PY_RETURN, LINE, RAISE ---
    990
    f_cdef PY_START [10], PY_RETURN [10], LINE [10]
    f_cpdef PY_START [20], PY_RETURN [20], LINE [20]
    f_def PY_START [10], PY_RETURN [10], LINE [10]
    f_inline PY_START [10], PY_RETURN [10], LINE [10]
    f_inline_prof PY_START [10], PY_RETURN [10], LINE [10]
    f_nogil_prof PY_START [10], PY_RETURN [10], LINE [10]
    f_raise PY_START [20], LINE [20], RAISE [20], PY_UNWIND [20]
    f_reraise PY_START [10], LINE [50], RAISE [10], PY_UNWIND [10], RERAISE [10]
    f_return_default PY_START [10], PY_RETURN [10], LINE [10]
    f_return_none PY_START [10], PY_RETURN [10], LINE [10]
    f_withgil_prof PY_START [10], PY_RETURN [10], LINE [10]
    m_cdef PY_START [10], PY_RETURN [10], LINE [10]
    m_classmethod PY_START [10], PY_RETURN [10], LINE [10]
    m_cpdef PY_START [20], PY_RETURN [20], LINE [20]
    m_def PY_START [20], PY_RETURN [20], LINE [20]
    m_staticmethod PY_START [10], PY_RETURN [10], LINE [10]
    test_profile PY_START [1], PY_RETURN [1], LINE [385], RAISE [20]


    ## Testing fused functions:

    >>> with monitored_events(events=FUNC_EVENTS, function_name='call_fused_functions') as collected_events:
    ...     call_fused_functions()
    >>> print_events(collected_events)  # call_fused_functions()
    call_fused_functions PY_START [1], PY_RETURN [1]


    ## Testing special return types:

    >>> events = set()
    >>> def trace_return(code, offset, arg):
    ...     events.add(arg)

    >>> _ = smon.register_callback(TOOL_ID, E.PY_RETURN, trace_return)
    >>> smon.set_events(TOOL_ID, E.PY_RETURN)
    >>> try:
    ...     # monitored
    ...     trace_return_neg_1()
    ...     trace_return_charptr()
    ... finally:
    ...     _ = smon.register_callback(TOOL_ID, E.PY_RETURN, None)
    ...     smon.free_tool_id(TOOL_ID)
    -1
    b'xyzxyz'

    >>> -1 in events  or  events
    True
    >>> None in events  or  events
    True
    >>> b'xyzxyz' in events  or  events
    True
    >>> (b'xyz' not in events if COMPILED else b'xyz' in events)  or  events
    True


    >>> smon.free_tool_id(TOOL_ID)
    """


def names(event_ids):
    return ', '.join([event_name(1 << event_id) for event_id in sorted(event_ids)])


def print_events(collected_events):
    def event_sum(event_id, event_counts):
        count = sum(event_counts)
        if not cython.compiled and event_id == E.RERAISE and count == 20:
            # CPython bug, see https://github.com/python/cpython/issues/118360
            count //= 2
        return count

    for func_name, line_counts in sorted(collected_events.items()):
        print(func_name,
            ', '.join(f"{names([event_id])} [{event_sum(event_id, counts.values())}]" for event_id, counts in sorted(line_counts.items())))


def assert_events(expected_events, collected_events, loops=10):
    missing = set(expected_events).difference(collected_events)
    surplus = set(collected_events).difference(expected_events)

    if missing:
        print(f"Expected: [{names(expected_events)}], missing: [{names(missing)}]")
    if surplus:
        print(f"Expected: [{names(expected_events)}], surplus: [{names(surplus)}]")

    collected_events = {name: dict(values) for name, values in collected_events.items()}


@contextmanager
@cython.profile(False)
@cython.linetrace(False)
def monitored_events(events=FUNC_EVENTS, function_name="test_profile"):
    event_set = E.NO_EVENTS
    for event in events:
        event_set |= event

    collected_events = defaultdict(lambda: defaultdict(lambda: defaultdict(int)))
    collected_line_events = defaultdict(int)

    @cython.profile(False)
    @cython.linetrace(False)
    def count_event(event, code_obj, offset, *args):
        if not (code_obj.co_name.startswith('f_') or code_obj.co_name.startswith('m_') or code_obj.co_name == function_name):
            return smon.DISABLE if event != E.RAISE else None
        if ' (wrapper)' in code_obj.co_name:
            return  # ignore cpdef wrapper
        if not cython.compiled and 'noprof' in code_obj.co_name:
            return smon.DISABLE if event != E.RAISE else None
        if event == E.LINE:
            # offset == line
            collected_line_events[offset] += 1
            assert offset in (line for line, *_ in code_obj.co_positions()), f"{code_obj.co_name}: {offset} in {list(code_obj.co_positions())}"
        collected_events[code_obj.co_name][event][offset] += 1

    try:
        for event in events:
            smon.register_callback(TOOL_ID, event, partial(count_event, event))
        smon.set_events(TOOL_ID, event_set)
        if event_set & E.LINE:
            yield collected_events, collected_line_events
        else:
            yield collected_events
    finally:
        smon.set_events(TOOL_ID, event_set)
        for event in events:
            smon.register_callback(TOOL_ID, event, None)


####### Traced code


def test_profile(N: cython.long):
    i: cython.long
    n: cython.long = 0
    obj: object
    a: CyA = CyA()
    py_a = PyA()

    for i in range(N):
        n += f_def(i)
        n += f_cdef(i)
        n += f_cpdef(i)
        obj = f_cpdef
        n += obj(i)

        n += f_inline(i)
        n += f_inline_prof(i)
        n += f_noprof(i)
        n += f_nogil_noprof(i)
        n += f_nogil_prof(i)
        n += f_withgil_noprof(i)
        n += f_withgil_prof(i)

        retval = f_return_none(i)
        assert retval is None, retval
        retval = f_return_default(i)
        assert retval == (0 if cython.compiled else None), retval

        n += a.m_def(i)
        obj = a
        n += obj.m_def(i)
        n += a.m_cpdef(i)
        n += obj.m_cpdef(i)
        n += a.m_cdef(i)
        n += CyA.m_staticmethod(i)
        n += a.m_classmethod(i)

        n += py_a.pym_def(i)
        obj = py_a
        n += obj.pym_def(i)
        n += py_a.pym_staticmethod(i)
        n += py_a.pym_classmethod(i)

        try:
            n += f_raise(i+2)
        except RuntimeError:
            pass
        try:
            n += f_reraise(i+2)
        except RuntimeError:
            pass
    return n

def f_def(a: cython.long):
    return a

@cython.cfunc
def f_cdef(a: cython.long) -> cython.long:
    return a

@cython.ccall
def f_cpdef(a: cython.long) -> cython.long:
    return a

@cython.inline
@cython.cfunc
def f_inline(a: cython.long) -> cython.long:
    return a

@cython.profile(True)
@cython.inline
@cython.cfunc
def f_inline_prof(a: cython.long) -> cython.long:
    return a

@cython.profile(False)
@cython.linetrace(False)
@cython.cfunc
def f_noprof(a: cython.long) -> cython.long:
    return a

@cython.exceptval(-2)
@cython.cfunc
def f_raise(_: cython.long) -> cython.long:
    raise RuntimeError

@cython.exceptval(-2)
@cython.cfunc
def f_reraise(a: cython.long) -> cython.long:
    try:
        f_raise(a)
    except ValueError:
        pass
    except RuntimeError:
        raise

@cython.profile(False)
@cython.linetrace(False)
@cython.with_gil
@cython.cfunc
def f_withgil_noprof(a: cython.long) -> cython.long:
    return a

@cython.profile(True)
@cython.with_gil
@cython.cfunc
def f_withgil_prof(a: cython.long) -> cython.long:
    return a

@cython.profile(False)
@cython.linetrace(False)
@cython.nogil
@cython.cfunc
def f_nogil_noprof(a: cython.long) -> cython.long:
    return a

@cython.profile(True)
@cython.nogil
@cython.cfunc
def f_nogil_prof(a: cython.long) -> cython.long:
    return a

@cython.ccall
def f_return_none(_: cython.long):
    pass

@cython.ccall
def f_return_default(_: cython.long) -> cython.long:
    pass


@cython.cclass
class CyA:
    def m_def(self, a: cython.long):
        return a

    @cython.ccall
    def m_cpdef(self, a: cython.long):
        return a

    @cython.cfunc
    def m_cdef(self, a: cython.long) -> cython.long:
        return a

    @classmethod
    @cython.cfunc
    def m_classmethod(cls, a: cython.long) -> cython.long:
        return a

    @staticmethod
    @cython.cfunc
    def m_staticmethod(a: cython.long) -> cython.long:
        return a


class PyA:
    def pym_def(self, a: cython.long):
        return a

    @classmethod
    def pym_classmethod(cls, a: cython.long) -> cython.long:
        return a

    @staticmethod
    def pym_staticmethod(a: cython.long) -> cython.long:
        return a


# Generators

def test_generators(_=None):
    call_generator()
    call_generator_exception()
    f_generator_expr()

def call_generator():
    list(f_generator())

def f_generator():
    yield 1
    yield 2

def call_generator_exception():
    try:
        list(f_generator_exception())
    except ValueError:
        pass

def f_generator_exception():
    yield 1
    raise ValueError(2)

def f_generator_expr():
    e = (x for x in range(10))
    return sum(e)


# Fused functions

def call_fused_functions():
    if cython.compiled:
        assert 5 == fused_func_def['short'](3)
        assert 5 == fused_func_def['int'](3)
        assert 5 == fused_func_def['double'](3)
        assert 5 == fused_func_def['float'](3)
        assert 5 == fused_func_def['long'](3)

    assert 5.0 == fused_func_def(3.0)
    assert 5 == fused_func_def(3)

    if cython.compiled:
        assert 5 == fused_func_cfunc[cython.short](3)
        assert 5 == fused_func_cfunc[cython.int](3)
        assert 5 == fused_func_cfunc[cython.double](3)
        assert 5 == fused_func_cfunc[cython.float](3)
        assert 5 == fused_func_cfunc[cython.long](3)

    assert 5.0 == fused_func_cfunc(3.0)
    assert 5 == fused_func_cfunc(3)


def fused_func_def(x: cython.numeric) -> cython.numeric:
    return x + 2

@cython.cfunc
def fused_func_cfunc(x: cython.numeric) -> cython.numeric:
    return x + 2


# Special return values

@cython.cfunc
def c_return_neg_1() -> cython.Py_UCS4:
    # Returning an invalid Py_UCS4 value could fail in combination with trace result reporting.
    # See https://github.com/cython/cython/issues/6503
    return cython.cast(cython.Py_UCS4, -1)

def call_c_return_neg_1():
    return cython.cast(cython.int, c_return_neg_1())

def trace_return_neg_1():
    result = call_c_return_neg_1()
    return result


@cython.cfunc
def c_return_charptr() -> cython.p_char:
    return b"xyz"

def trace_return_charptr():
    result: object = c_return_charptr()
    return result * 2



# distutils: language = c++

from cython.cimports.libcpp.vector import vector

@cython.cclass
class Matrix:
    ncols: cython.uint
    v: vector[cython.float]

    def __cinit__(self, ncols: cython.uint):
        self.ncols = ncols

    def add_row(self):
        """Adds a row, initially zero-filled."""
        self.v.resize(self.v.size() + self.ncols)



from libc.stdio cimport FILE
cimport cpython.type

cdef extern from "Python.h":

    ctypedef struct PyObject  # forward declaration

    ctypedef object (*newfunc)(cpython.type.type, PyObject*, PyObject*)  # (type, args|NULL, kwargs|NULL)

    ctypedef object (*unaryfunc)(object)
    ctypedef object (*binaryfunc)(object, object)
    ctypedef object (*ternaryfunc)(object, object, object)
    ctypedef int (*inquiry)(object) except -1
    ctypedef Py_ssize_t (*lenfunc)(object) except -1
    ctypedef object (*ssizeargfunc)(object, Py_ssize_t)
    ctypedef object (*ssizessizeargfunc)(object, Py_ssize_t, Py_ssize_t)
    ctypedef int (*ssizeobjargproc)(object, Py_ssize_t, object) except -1
    ctypedef int (*ssizessizeobjargproc)(object, Py_ssize_t, Py_ssize_t, object) except -1
    ctypedef int (*objobjargproc)(object, object, object) except -1
    ctypedef int (*objobjproc)(object, object) except -1

    ctypedef Py_hash_t (*hashfunc)(object) except -1
    ctypedef object (*reprfunc)(object)

    ctypedef int (*cmpfunc)(object, object) except -2
    ctypedef object (*richcmpfunc)(object, object, int)

    # The following functions use 'PyObject*' as first argument instead of 'object' to prevent
    # accidental reference counting when calling them during a garbage collection run.
    ctypedef void (*destructor)(PyObject*)
    ctypedef int (*visitproc)(PyObject*, void *) except -1
    ctypedef int (*traverseproc)(PyObject*, visitproc, void*) except -1
    ctypedef void (*freefunc)(void*)

    ctypedef object (*descrgetfunc)(object, object, object)
    ctypedef int (*descrsetfunc)(object, object, object) except -1

    ctypedef object (*PyCFunction)(object, object)

    ctypedef struct PyMethodDef:
        const char* ml_name
        PyCFunction ml_meth
        int ml_flags
        const char* ml_doc

    ctypedef struct PyTypeObject:
        const char* tp_name
        const char* tp_doc
        Py_ssize_t tp_basicsize
        Py_ssize_t tp_itemsize
        Py_ssize_t tp_dictoffset
        unsigned long tp_flags

        newfunc tp_new
        destructor tp_dealloc
        destructor tp_del
        destructor tp_finalize
        traverseproc tp_traverse
        inquiry tp_clear
        freefunc tp_free

        ternaryfunc tp_call
        hashfunc tp_hash
        reprfunc tp_str
        reprfunc tp_repr

        cmpfunc tp_compare
        richcmpfunc tp_richcompare

        PyMethodDef* tp_methods

        PyTypeObject* tp_base
        PyObject* tp_dict

        descrgetfunc tp_descr_get
        descrsetfunc tp_descr_set

        unsigned int tp_version_tag

    ctypedef struct PyObject:
        Py_ssize_t ob_refcnt
        PyTypeObject *ob_type

    cdef PyTypeObject *Py_TYPE(object)

    void* PyObject_Malloc(size_t)
    void* PyObject_Realloc(void *, size_t)
    void PyObject_Free(void *)

    #####################################################################
    # 6.1 Object Protocol
    #####################################################################
    int PyObject_Print(object o, FILE *fp, int flags) except -1
    # Print an object o, on file fp. Returns -1 on error. The flags
    # argument is used to enable certain printing options. The only
    # option currently supported is Py_PRINT_RAW; if given, the str()
    # of the object is written instead of the repr().

    bint PyObject_HasAttrString(object o, const char *attr_name)
    # Returns 1 if o has the attribute attr_name, and 0
    # otherwise. This is equivalent to the Python expression
    # "hasattr(o, attr_name)". This function always succeeds.

    object PyObject_GetAttrString(object o, const char *attr_name)
    # Return value: New reference.  Retrieve an attribute named
    # attr_name from object o. Returns the attribute value on success,
    # or NULL on failure. This is the equivalent of the Python
    # expression "o.attr_name".

    bint PyObject_HasAttr(object o, object attr_name)
    # Returns 1 if o has the attribute attr_name, and 0
    # otherwise. This is equivalent to the Python expression
    # "hasattr(o, attr_name)". This function always succeeds.

    object PyObject_GetAttr(object o, object attr_name)
    # Return value: New reference.  Retrieve an attribute named
    # attr_name from object o. Returns the attribute value on success,
    # or NULL on failure. This is the equivalent of the Python
    # expression "o.attr_name".

    object PyObject_GenericGetAttr(object o, object attr_name)

    int PyObject_SetAttrString(object o, const char *attr_name, object v) except -1
    # Set the value of the attribute named attr_name, for object o, to
    # the value v. Returns -1 on failure. This is the equivalent of
    # the Python statement "o.attr_name = v".

    int PyObject_SetAttr(object o, object attr_name, object v) except -1
    # Set the value of the attribute named attr_name, for object o, to
    # the value v. Returns -1 on failure. This is the equivalent of
    # the Python statement "o.attr_name = v".

    int PyObject_GenericSetAttr(object o, object attr_name, object v) except -1

    int PyObject_DelAttrString(object o, const char *attr_name) except -1
    # Delete attribute named attr_name, for object o. Returns -1 on
    # failure. This is the equivalent of the Python statement: "del
    # o.attr_name".

    int PyObject_DelAttr(object o, object attr_name) except -1
    # Delete attribute named attr_name, for object o. Returns -1 on
    # failure. This is the equivalent of the Python statement "del
    # o.attr_name".

    object PyObject_GenericGetDict(object o, void *context)
    # Return value: New reference.
    # A generic implementation for the getter of a __dict__ descriptor. It
    # creates the dictionary if necessary.
    # New in version 3.3.

    int PyObject_GenericSetDict(object o, object value, void *context) except -1
    # A generic implementation for the setter of a __dict__ descriptor. This
    # implementation does not allow the dictionary to be deleted.
    # New in version 3.3.

    int Py_LT, Py_LE, Py_EQ, Py_NE, Py_GT, Py_GE

    object PyObject_RichCompare(object o1, object o2, int opid)
    # Return value: New reference.
    # Compare the values of o1 and o2 using the operation specified by
    # opid, which must be one of Py_LT, Py_LE, Py_EQ, Py_NE, Py_GT, or
    # Py_GE, corresponding to <, <=, ==, !=, >, or >=
    # respectively. This is the equivalent of the Python expression
    # "o1 op o2", where op is the operator corresponding to
    # opid. Returns the value of the comparison on success, or NULL on
    # failure.

    bint PyObject_RichCompareBool(object o1, object o2, int opid) except -1
    # Compare the values of o1 and o2 using the operation specified by
    # opid, which must be one of Py_LT, Py_LE, Py_EQ, Py_NE, Py_GT, or
    # Py_GE, corresponding to <, <=, ==, !=, >, or >=
    # respectively. Returns -1 on error, 0 if the result is false, 1
    # otherwise. This is the equivalent of the Python expression "o1
    # op o2", where op is the operator corresponding to opid.

    int PyObject_Cmp(object o1, object o2, int *result) except -1
    # Compare the values of o1 and o2 using a routine provided by o1,
    # if one exists, otherwise with a routine provided by o2. The
    # result of the comparison is returned in result. Returns -1 on
    # failure. This is the equivalent of the Python statement "result
    # = cmp(o1, o2)".

    int PyObject_Compare(object o1, object o2) except *
    # Compare the values of o1 and o2 using a routine provided by o1,
    # if one exists, otherwise with a routine provided by o2. Returns
    # the result of the comparison on success. On error, the value
    # returned is undefined; use PyErr_Occurred() to detect an
    # error. This is equivalent to the Python expression "cmp(o1,
    # o2)".

    object PyObject_Repr(object o)
    # Return value: New reference.
    # Compute a string representation of object o. Returns the string
    # representation on success, NULL on failure. This is the
    # equivalent of the Python expression "repr(o)". Called by the
    # repr() built-in function and by reverse quotes.

    object PyObject_Str(object o)
    # Return value: New reference.
    # Compute a string representation of object o. Returns the string
    # representation on success, NULL on failure. This is the
    # equivalent of the Python expression "str(o)". Called by the
    # str() built-in function and by the print statement.

    object PyObject_Bytes(object o)
    # Return value: New reference.
    # Compute a bytes representation of object o. Return NULL on
    # failure and a bytes object on success. This is equivalent to
    # the Python expression bytes(o), when o is not an integer.
    # Unlike bytes(o), a TypeError is raised when o is an integer
    # instead of a zero-initialized bytes object.

    bint PyObject_IsInstance(object inst, object cls) except -1
    # Returns 1 if inst is an instance of the class cls or a subclass
    # of cls, or 0 if not. On error, returns -1 and sets an
    # exception. If cls is a type object rather than a class object,
    # PyObject_IsInstance() returns 1 if inst is of type cls. If cls
    # is a tuple, the check will be done against every entry in
    # cls. The result will be 1 when at least one of the checks
    # returns 1, otherwise it will be 0. If inst is not a class
    # instance and cls is neither a type object, nor a class object,
    # nor a tuple, inst must have a __class__ attribute -- the class
    # relationship of the value of that attribute with cls will be
    # used to determine the result of this function.

    # Subclass determination is done in a fairly straightforward way,
    # but includes a wrinkle that implementors of extensions to the
    # class system may want to be aware of. If A and B are class
    # objects, B is a subclass of A if it inherits from A either
    # directly or indirectly. If either is not a class object, a more
    # general mechanism is used to determine the class relationship of
    # the two objects. When testing if B is a subclass of A, if A is
    # B, PyObject_IsSubclass() returns true. If A and B are different
    # objects, B's __bases__ attribute is searched in a depth-first
    # fashion for A -- the presence of the __bases__ attribute is
    # considered sufficient for this determination.

    bint PyObject_IsSubclass(object derived, object cls) except -1
    # Returns 1 if the class derived is identical to or derived from
    # the class cls, otherwise returns 0. In case of an error, returns
    # -1. If cls is a tuple, the check will be done against every
    # entry in cls. The result will be 1 when at least one of the
    # checks returns 1, otherwise it will be 0. If either derived or
    # cls is not an actual class object (or tuple), this function uses
    # the generic algorithm described above. New in version
    # 2.1. Changed in version 2.3: Older versions of Python did not
    # support a tuple as the second argument.

    bint PyCallable_Check(object o)
    # Determine if the object o is callable. Return 1 if the object is
    # callable and 0 otherwise. This function always succeeds.

    object PyObject_Call(object callable_object, object args, object kw)
    # Return value: New reference.
    # Call a callable Python object callable_object, with arguments
    # given by the tuple args, and named arguments given by the
    # dictionary kw. If no named arguments are needed, kw may be
    # NULL. args must not be NULL, use an empty tuple if no arguments
    # are needed. Returns the result of the call on success, or NULL
    # on failure. This is the equivalent of the Python expression
    # "apply(callable_object, args, kw)" or "callable_object(*args,
    # **kw)".

    object PyObject_CallObject(object callable_object, object args)
    # Return value: New reference.
    # Call a callable Python object callable_object, with arguments
    # given by the tuple args. If no arguments are needed, then args
    # may be NULL. Returns the result of the call on success, or NULL
    # on failure. This is the equivalent of the Python expression
    # "apply(callable_object, args)" or "callable_object(*args)".

    object PyObject_CallFunction(object callable, char *format, ...)
    # Return value: New reference.
    # Call a callable Python object callable, with a variable number
    # of C arguments. The C arguments are described using a
    # Py_BuildValue() style format string. The format may be NULL,
    # indicating that no arguments are provided. Returns the result of
    # the call on success, or NULL on failure. This is the equivalent
    # of the Python expression "apply(callable, args)" or
    # "callable(*args)". Note that if you only pass object  args,
    # PyObject_CallFunctionObjArgs is a faster alternative.

    object PyObject_CallMethod(object o, char *method, char *format, ...)
    # Return value: New reference.
    # Call the method named method of object o with a variable number
    # of C arguments. The C arguments are described by a
    # Py_BuildValue() format string that should produce a tuple. The
    # format may be NULL, indicating that no arguments are
    # provided. Returns the result of the call on success, or NULL on
    # failure. This is the equivalent of the Python expression
    # "o.method(args)". Note that if you only pass object  args,
    # PyObject_CallMethodObjArgs is a faster alternative.

    #object PyObject_CallFunctionObjArgs(object callable, ..., NULL)
    object PyObject_CallFunctionObjArgs(object callable, ...)
    # Return value: New reference.
    # Call a callable Python object callable, with a variable number
    # of PyObject* arguments. The arguments are provided as a variable
    # number of parameters followed by NULL. Returns the result of the
    # call on success, or NULL on failure.

    #PyObject* PyObject_CallMethodObjArgs(object o, object name, ..., NULL)
    object PyObject_CallMethodObjArgs(object o, object name, ...)
    # Return value: New reference.
    # Calls a method of the object o, where the name of the method is
    # given as a Python string object in name. It is called with a
    # variable number of PyObject* arguments. The arguments are
    # provided as a variable number of parameters followed by
    # NULL. Returns the result of the call on success, or NULL on
    # failure.

    long PyObject_Hash(object o) except? -1
    # Compute and return the hash value of an object o. On failure,
    # return -1. This is the equivalent of the Python expression
    # "hash(o)".

    bint PyObject_IsTrue(object o) except -1
    # Returns 1 if the object o is considered to be true, and 0
    # otherwise. This is equivalent to the Python expression "not not
    # o". On failure, return -1.

    bint PyObject_Not(object o) except -1
    # Returns 0 if the object o is considered to be true, and 1
    # otherwise. This is equivalent to the Python expression "not
    # o". On failure, return -1.

    object PyObject_Type(object o)
    # Return value: New reference.
    # When o is non-NULL, returns a type object corresponding to the
    # object type of object o. On failure, raises SystemError and
    # returns NULL. This is equivalent to the Python expression
    # type(o). This function increments the reference count of the
    # return value. There's really no reason to use this function
    # instead of the common expression o->ob_type, which returns a
    # pointer of type PyTypeObject*, except when the incremented
    # reference count is needed.

    bint PyObject_TypeCheck(object o, PyTypeObject *type)
    # Return true if the object o is of type type or a subtype of
    # type. Both parameters must be non-NULL.

    Py_ssize_t PyObject_Length(object o) except -1
    Py_ssize_t PyObject_Size(object o) except -1
    # Return the length of object o. If the object o provides either
    # the sequence and mapping protocols, the sequence length is
    # returned. On error, -1 is returned. This is the equivalent to
    # the Python expression "len(o)".

    Py_ssize_t PyObject_LengthHint(object o, Py_ssize_t default) except -1
    # Return an estimated length for the object o. First try to return its
    # actual length, then an estimate using __length_hint__(), and finally
    # return the default value. On error, return -1. This is the equivalent to
    # the Python expression "operator.length_hint(o, default)".
    # New in version 3.4.

    object PyObject_GetItem(object o, object key)
    # Return value: New reference.
    # Return element of o corresponding to the object key or NULL on
    # failure. This is the equivalent of the Python expression
    # "o[key]".

    int PyObject_SetItem(object o, object key, object v) except -1
    # Map the object key to the value v. Returns -1 on failure. This
    # is the equivalent of the Python statement "o[key] = v".

    int PyObject_DelItem(object o, object key) except -1
    # Delete the mapping for key from o. Returns -1 on failure. This
    # is the equivalent of the Python statement "del o[key]".

    int PyObject_AsFileDescriptor(object o) except -1
    # Derives a file-descriptor from a Python object. If the object is
    # an integer or long integer, its value is returned. If not, the
    # object's fileno() method is called if it exists; the method must
    # return an integer or long integer, which is returned as the file
    # descriptor value. Returns -1 on failure.

    object PyObject_Dir(object o)
    # Return value: New reference.
    # This is equivalent to the Python expression "dir(o)", returning
    # a (possibly empty) list of strings appropriate for the object
    # argument, or NULL if there was an error. If the argument is
    # NULL, this is like the Python "dir()", returning the names of
    # the current locals; in this case, if no execution frame is
    # active then NULL is returned but PyErr_Occurred() will return
    # false.

    object PyObject_GetIter(object o)
    # Return value: New reference.
    # This is equivalent to the Python expression "iter(o)". It
    # returns a new iterator for the object argument, or the object
    # itself if the object is already an iterator. Raises TypeError
    # and returns NULL if the object cannot be iterated.

    Py_ssize_t Py_SIZE(object o)

    object PyObject_Format(object obj, object format_spec)
    # Takes an arbitrary object and returns the result of calling
    # obj.__format__(format_spec).
    # Added in Py2.6

    # Type flags (tp_flags of PyTypeObject)
    long Py_TPFLAGS_HAVE_GETCHARBUFFER
    long Py_TPFLAGS_HAVE_SEQUENCE_IN
    long Py_TPFLAGS_HAVE_INPLACEOPS
    long Py_TPFLAGS_CHECKTYPES
    long Py_TPFLAGS_HAVE_RICHCOMPARE
    long Py_TPFLAGS_HAVE_WEAKREFS
    long Py_TPFLAGS_HAVE_ITER
    long Py_TPFLAGS_HAVE_CLASS
    long Py_TPFLAGS_HEAPTYPE
    long Py_TPFLAGS_BASETYPE
    long Py_TPFLAGS_READY
    long Py_TPFLAGS_READYING
    long Py_TPFLAGS_HAVE_GC
    long Py_TPFLAGS_HAVE_STACKLESS_EXTENSION
    long Py_TPFLAGS_HAVE_INDEX
    long Py_TPFLAGS_HAVE_VERSION_TAG
    long Py_TPFLAGS_VALID_VERSION_TAG
    long Py_TPFLAGS_IS_ABSTRACT
    long Py_TPFLAGS_HAVE_NEWBUFFER
    long Py_TPFLAGS_INT_SUBCLASS
    long Py_TPFLAGS_LONG_SUBCLASS
    long Py_TPFLAGS_LIST_SUBCLASS
    long Py_TPFLAGS_TUPLE_SUBCLASS
    long Py_TPFLAGS_STRING_SUBCLASS
    long Py_TPFLAGS_UNICODE_SUBCLASS
    long Py_TPFLAGS_DICT_SUBCLASS
    long Py_TPFLAGS_BASE_EXC_SUBCLASS
    long Py_TPFLAGS_TYPE_SUBCLASS
    long Py_TPFLAGS_DEFAULT_EXTERNAL
    long Py_TPFLAGS_DEFAULT_CORE
    long Py_TPFLAGS_DEFAULT
    long Py_TPFLAGS_HAVE_FINALIZE



# mode: error
# tag: warnings

cdef extern from *:
    cdef void f()
    cdef void (*fp)() nogil
    ctypedef void (*fp_t)() nogil

fp = f
fp = <fp_t>f

_ERRORS = u"""
9:5: Cannot assign type 'void (void) noexcept' to 'void (*)(void) noexcept nogil'
"""

_WARNINGS = """
10:5: Casting a GIL-requiring function into a nogil function circumvents GIL validation
"""



# mode: run


cdef cfunc1(char* s):
    return s.decode('ASCII')


cdef cfunc3(int x, char* s, object y):
    return cfunc1(s)


def test_one_arg_indexing(s):
    """
    >>> test_one_arg_indexing(b'xyz')
    'y'
    """
    cfunc1(s[:1])
    z = cfunc1(s[2:])
    assert z == 'z', repr(z)
    return cfunc1(s[1:2])


def test_more_args_indexing(s):
    """
    >>> test_more_args_indexing(b'xyz')
    'y'
    """
    cfunc3(1, s[slice(0,1)], 6.5)
    z = cfunc3(2, s[slice(2,None)], 'abc' * 2)
    assert z == 'z', repr(z)
    return cfunc3(3, s[slice(1,2)], 1)


def test_one_arg_slicing(s):
    """
    >>> test_one_arg_slicing(b'xyz')
    'y'
    """
    cfunc1(s[:2])
    z = cfunc1(s[2:])
    assert z == 'z', repr(z)
    return cfunc1(s[1:2])


def test_more_args_slicing(s):
    """
    >>> test_more_args_slicing(b'xyz')
    'y'
    """
    cfunc3(1, s[:2], 'abc')
    z = cfunc3(123, s[2:], 5)
    assert z == 'z', repr(z)
    return cfunc3(2, s[1:2], 1.4)


def test_one_arg_adding(s):
    """
    >>> test_one_arg_adding(b'xyz')
    'abxyzqr'
    """
    return cfunc1(b"a" + b"b" + s + b"q" + b"r")


def test_more_args_adding(s):
    """
    >>> test_more_args_adding(b'xyz')
    'abxyzqr'
    """
    return cfunc3(1, b"a" + b"b" + s + b"q" + b"r", 'xyz%d' % 3)


cdef char* ret_charptr(char* s):
    return s


def test_charptr_and_charptr_func(char* s):
    """
    >>> test_charptr_and_charptr_func(b'abc') == b'abc'
    True
    """
    return s and ret_charptr(s)


def test_charptr_and_ucharptr(char* s):
    """
    >>> test_charptr_and_ucharptr(b'abc') == b'abc'
    True
    """
    return s and <unsigned char*>s



# Some comments first


# More comments

'A module docstring'

doctest = u"""# Python 3 gets all of these right ...
    >>> __doc__
    'A module docstring'

    >>> f.__doc__
    '\\n    This is a function docstring.\\n    '

    >>> C.__doc__
    '\\n    This is a class docstring.\\n    '
    >>> C.docstring_copy_C
    '\\n    This is a class docstring.\\n    '
    >>> CS.docstring_copy_C
    '\\n    This is a class docstring.\\n    '

    >>> CS.__doc__
    '\\n    This is a subclass docstring.\\n    '
    >>> CS.docstring_copy_CS
    '\\n    This is a subclass docstring.\\n    '
    >>> CSS.docstring_copy_CS
    '\\n    This is a subclass docstring.\\n    '
    >>> print(CSS.__doc__)
    None
    >>> CSS.docstring_copy_CSS
    'A module docstring'

    >>> T.__doc__
    '\\n    This is an extension type docstring.\\n    '
    >>> TS.__doc__
    '\\n    This is an extension subtype docstring.\\n    '
    >>> TSS.__doc__

Compare with standard Python:

    >>> def Pyf():
    ...     '''
    ...     This is a function docstring.
    ...     '''
    >>> Pyf.__doc__.strip()  # .strip() is needed because Py3.13 removes the indentation.
    'This is a function docstring.'

    >>> class PyC(object):
    ...     '''
    ...     This is a class docstring.
    ...     '''
    ...     docstring_copy_C = __doc__
    >>> class PyCS(PyC):
    ...     '''
    ...     This is a subclass docstring.
    ...     '''
    ...     docstring_copy_CS = __doc__
    >>> class PyCSS(PyCS):
    ...     docstring_copy_CSS = __doc__

    >>> PyC.__doc__.strip()
    'This is a class docstring.'
    >>> PyC.docstring_copy_C.strip()
    'This is a class docstring.'
    >>> PyCS.docstring_copy_C.strip()
    'This is a class docstring.'
    >>> PyCSS.docstring_copy_C.strip()
    'This is a class docstring.'

    >>> PyCS.__doc__.strip()
    'This is a subclass docstring.'
    >>> PyCS.docstring_copy_CS.strip()
    'This is a subclass docstring.'
    >>> PyCSS.docstring_copy_CS.strip()
    'This is a subclass docstring.'

    >>> PyCSS.__doc__
    >>> PyCSS.docstring_copy_CSS
    'A module docstring'
"""

__test__ = {"test_docstrings" : doctest}

def f():
    """
    This is a function docstring.
    """


class C(object):
    """
    This is a class docstring.
    """
    docstring_copy_C = __doc__


class CS(C):
    """
    This is a subclass docstring.
    """
    docstring_copy_CS = __doc__


class CSS(CS):
    docstring_copy_CSS = __doc__


cdef class T:
    """
    This is an extension type docstring.
    """

cdef class TS(T):
    """
    This is an extension subtype docstring.
    """

cdef class TSS(TS):
    pass


def n():
    "This is not a docstring".lower()

class PyN(object):
    u"This is not a docstring".lower()

cdef class CN(object):
    b"This is not a docstring".lower()


def test_non_docstrings():
    """
    >>> n.__doc__
    >>> PyN.__doc__
    >>> CN.__doc__
    """



# mode: run
# tag: cpp, no-cpp-locals

cdef extern from *:
    """
    class Base {
    public:
        int foo(int a) {
            return a + 1;
        }
    };
    """
    cdef cppclass Base:
        __init__()
        int foo(int)

cdef cppclass Derived(Base):
    # We need to specify noexcept here because Base.foo(int) is by default wrapped as noexcept.
    int foo(int a) noexcept:
        return a * 2;

cdef cppclass DefinedBase:
    int state

    __init__():
        # If we skip this one, inheritance won't work because of the
        # derived class constructor implicitly calling default constructor
        this.__init__(0)

    __init__(int a):
        this.state = a

    int getter():
        return this.state

    int foo(int a):
        return this.state + a

cdef cppclass DefinedDerived(DefinedBase):
    __init__(int a):
        DefinedBase.__init__(2*a)

    int foo(int a):
        return this.state * a

def testDeclared():
    """
    >>> testDeclared()
    5 8
    """
    cdef int a = 4
    cdef Base b
    cdef Derived d
    rst_a = b.foo(a)
    rst_b = d.foo(a)
    print rst_a, rst_b

def testDefined():
    """
    >>> testDefined()
    24 34 48 480
    """

    db = new DefinedBase(24)
    dd = new DefinedDerived(24)

    rst_a = db.getter()
    rst_b = db.foo(10)
    rst_c = dd.getter()
    rst_d = dd.foo(10)

    print rst_a, rst_b, rst_c, rst_d
    del db
    del dd



# mode: error

cimport cython

def f():
    o = object()
    with cython.critical_section():  # no arguments
        pass
    with cython.critical_section(o, o, o):  # Too many arguments (also the same argument, but we don't check for that)
        pass
    with cython.critical_section(please_ensure_thread_safety=True):  # While this keyword argument would be nice, it doesn't exist
        pass
    with cython.critical_section(1):  # C argument type
        pass
    with cython.critical_section(<void*>o):  # silly, but again, a C argument
        pass
    with nogil:
        with cython.critical_section(o):  # critical sections require the GIL
            pass

@cython.critical_section
def crit_sec_func():
    pass

@cython.critical_section
def crit_sec_func2(*args):
    pass

@cython.critical_section
def crit_sec_func3(**kwds):
    pass

@cython.critical_section(True)  # should not have arguments
def crit_sec_func4(arg):
    pass

@cython.critical_section
def has_int_arg(int arg):
    pass

@cython.critical_section
cdef void some_nogil_function(a) noexcept nogil:
    pass


_ERRORS = """
7:9: critical_section directive accepts one or two positional arguments
9:9: critical_section directive accepts one or two positional arguments
11:9: critical_section directive accepts one or two positional arguments
13:33: Arguments to cython.critical_section must be Python objects.
15:33: Arguments to cython.critical_section must be Python objects.
18:13: Critical sections require the GIL
21:0: critical_section directive can only be applied to a function with one or more positional arguments
25:0: critical_section directive can only be applied to a function with one or more positional arguments
29:0: critical_section directive can only be applied to a function with one or more positional arguments
33:0: critical_section decorator does not take arguments
37:0: Arguments to cython.critical_section must be Python objects.
41:0: Critical sections require the GIL

# slightly extraneous, but no real harm
18:37: Creating temporary Python reference not allowed without gil
41:0: Creating temporary Python reference not allowed without gil
"""



# mode: run

import cython

try:
    import typing
    from typing import List, Tuple
    from typing import Set as _SET_
except:
    pass  # this should allow Cython to interpret the directives even when the module doesn't exist

def test_subscripted_types():
    """
    >>> test_subscripted_types()
    dict object
    list object
    set object
    """
    cdef typing.Dict[int, float] a = {}
    cdef List[int] b = []
    cdef _SET_[object] c = set()

    print(cython.typeof(a))
    print(cython.typeof(b))
    print(cython.typeof(c))

cdef class TestClassVar:
    """
    >>> TestClassVar.cls
    5
    >>> TestClassVar.regular  # doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
        ...
    AttributeError:
    """
    cdef int regular
    cdef typing.ClassVar[int] cls
    cls = 5

# because tuple is specifically special cased to go to ctuple where possible
def test_tuple(typing.Tuple[int, float] a,  typing.Tuple[int, ...] b,
               Tuple[int, object] c  # cannot be a ctuple
               ):
    """
    >>> test_tuple((1, 1.0), (1, 1.0), (1, 1.0))
    int
    int
    tuple object
    tuple object
    """
    cdef typing.Tuple[int, float] x = (a[0], a[1])  # C int/float
    cdef Tuple[int, ...] y = (1,2.)
    z = a[0]  # should infer to C int

    print(cython.typeof(z))
    print(cython.typeof(x[0]))
    print(cython.typeof(y))
    print(cython.typeof(c))



__doc__ = """
    >>> s = Spam()
    >>> s.get_tons()
    17
    >>> s.set_tons(42)
    >>> s.get_tons()
    42
"""

import platform
if not hasattr(platform, 'python_implementation') or platform.python_implementation() == 'CPython':
    __doc__ += """
    >>> s = None
    42 tons of spam is history.
"""

cdef class Spam:

    cdef int tons

    def __cinit__(self):
        self.tons = 17

    def __dealloc__(self):
        print self.tons, u"tons of spam is history."

    def get_tons(self):
        return self.tons

    def set_tons(self, x):
        self.tons = x



from sin_of_square cimport Function, SinOfSquareFunction

def integrate(Function f, double a, double b, int N):
    cdef int i
    cdef double s, dx
    if f is None:
        raise ValueError("f cannot be None")

    s = 0
    dx = (b - a) / N

    for i in range(N):
        s += f.evaluate(a + i * dx)

    return s * dx

print(integrate(SinOfSquareFunction(), 0, 1, 10000))



# mode: run
# tag: pep557, pure3.7

import dataclasses
from typing import Sequence


@dataclasses.dataclass
class Color:
    """
    >>> list(Color.__dataclass_fields__.keys())
    ['red', 'green', 'blue', 'alpha']
    >>> Color(1, 2, 3)
    Color(red=1, green=2, blue=3, alpha=255)
    >>> Color(1, 2, 3, 4)
    Color(red=1, green=2, blue=3, alpha=4)
    >>> Color(green=1, blue=2, red=3, alpha=40)
    Color(red=3, green=1, blue=2, alpha=40)
    """
    red: int
    green: int
    blue: int
    alpha: int = 255


@dataclasses.dataclass
class NamedColor(Color):
    """
    >>> list(NamedColor.__dataclass_fields__.keys())
    ['red', 'green', 'blue', 'alpha', 'names']
    >>> NamedColor(1, 2, 3)
    NamedColor(red=1, green=2, blue=3, alpha=255, names=[])
    >>> NamedColor(1, 2, 3, 4)
    NamedColor(red=1, green=2, blue=3, alpha=4, names=[])
    >>> NamedColor(green=1, blue=2, red=3, alpha=40)
    NamedColor(red=3, green=1, blue=2, alpha=40, names=[])
    >>> NamedColor(1, 2, 3, names=["blackish", "very dark cyan"])
    NamedColor(red=1, green=2, blue=3, alpha=255, names=['blackish', 'very dark cyan'])
    """
    names: Sequence[str] = dataclasses.field(default_factory=list)


@dataclasses.dataclass(frozen=True)
class IceCream:
    """
    >>> IceCream("vanilla")
    IceCream(flavour='vanilla', num_toppings=2)
    >>> IceCream("vanilla") == IceCream("vanilla", num_toppings=3)
    False
    >>> IceCream("vanilla") == IceCream("vanilla", num_toppings=2)
    True
    """
    flavour: str
    num_toppings: int = 2



import cython

@cython.cclass
class Parrot:

    @cython.cfunc
    def describe(self) -> cython.void:
        print("This parrot is resting.")

@cython.cclass
class Norwegian(Parrot):

    @cython.cfunc
    def describe(self) -> cython.void:
        Parrot.describe(self)
        print("Lovely plumage!")

cython.declare(p1=Parrot, p2=Parrot)
p1 = Parrot()
p2 = Norwegian()
print("p2:")
p2.describe()



import cython
from cython.cimports.libc.stdlib import malloc, free

# Example C struct
my_c_struct = cython.struct(
    a = cython.int,
    b = cython.int,
)

@cython.cclass
class WrapperClass:
    """A wrapper class for a C/C++ data structure"""
    _ptr: cython.pointer[my_c_struct]
    ptr_owner: cython.bint

    def __cinit__(self):
        self.ptr_owner = False

    def __dealloc__(self):
        # De-allocate if not null and flag is set
        if self._ptr is not cython.NULL and self.ptr_owner is True:
            free(self._ptr)
            self._ptr = cython.NULL

    def __init__(self):
        # Prevent accidental instantiation from normal Python code
        # since we cannot pass a struct pointer into a Python constructor.
        raise TypeError("This class cannot be instantiated directly.")

    # Extension class properties
    @property
    def a(self):
        return self._ptr.a if self._ptr is not cython.NULL else None

    @property
    def b(self):
        return self._ptr.b if self._ptr is not cython.NULL else None

    @staticmethod
    @cython.cfunc
    def from_ptr(_ptr: cython.pointer[my_c_struct], owner: cython.bint=False) -> WrapperClass:
        """Factory function to create WrapperClass objects from
        given my_c_struct pointer.

        Setting ``owner`` flag to ``True`` causes
        the extension type to ``free`` the structure pointed to by ``_ptr``
        when the wrapper object is deallocated."""
        # Fast call to __new__() that bypasses the __init__() constructor.
        wrapper: WrapperClass  = WrapperClass.__new__(WrapperClass)
        wrapper._ptr = _ptr
        wrapper.ptr_owner = owner
        return wrapper

    @staticmethod
    @cython.cfunc
    def new_struct() -> WrapperClass:
        """Factory function to create WrapperClass objects with
        newly allocated my_c_struct"""
        _ptr: cython.pointer[my_c_struct] = cython.cast(
                cython.pointer[my_c_struct], malloc(cython.sizeof(my_c_struct)))
        if _ptr is cython.NULL:
            raise MemoryError
        _ptr.a = 0
        _ptr.b = 0
        return WrapperClass.from_ptr(_ptr, owner=True)



def test_in(s):
    """
    >>> test_in('ABC')
    1
    >>> test_in('abc')
    2
    >>> test_in('X')
    3
    >>> test_in('XYZ')
    4
    >>> test_in('ABCXYZ')
    5
    >>> test_in('')
    5
    """
    if s in (u'ABC', u'BCD', u'ABC'[:3], u'ABC'[::-1], u'ABC'[-1]):
        return 1
    elif s.upper() in (u'ABC', u'BCD'):
        return 2
    elif len(s) in (1,2):
        return 3
    elif len(s) in (3,4):
        return 4
    else:
        return 5

def test_not_in(s):
    """
    >>> test_not_in('abc')
    1
    >>> test_not_in('CDE')
    2
    >>> test_not_in('CDEF')
    3
    >>> test_not_in('BCD')
    4
    """
    if s not in (u'ABC', u'BCD', u'CDE', u'CDEF'):
        return 1
    elif s.upper() not in (u'ABC', u'BCD', u'CDEF'):
        return 2
    elif len(s) not in [3]:
        return 3
    elif len(s) not in [1,2]:
        return 4
    else:
        return 5



# cython: language_level=3
# mode: run
# tag: pure3.7, pep484, warnings

# for the benefit of the pure tests, don't require annotations
# to be evaluated
from __future__ import annotations

import typing
from typing import Tuple
import cython


def test_optional_ctuple(x: typing.Optional[tuple[float]]):
    """
    Should not be a C-tuple (because these can't be optional)
    >>> test_optional_ctuple((1.0,))
    tuple object
    """
    print(cython.typeof(x) + (" object" if not cython.compiled else ""))


def test_union_ctuple(x: typing.Union[tuple[float], None]):
    """
    Should not be a C-tuple (because these can't be optional)
    >>> test_union_ctuple((1.0,))
    tuple object
    """
    print(cython.typeof(x) + (" object" if not cython.compiled else ""))


def test_bitwise_or_ctuple(x: tuple[float] | None, y: None | tuple[float]):
    """
    Should not be a C-tuple (because these can't be optional)
    >>> test_bitwise_or_ctuple((1.0,), (2.0,))
    tuple object
    tuple object
    """
    print(cython.typeof(x) + (" object" if not cython.compiled else ""))
    print(cython.typeof(y) + (" object" if not cython.compiled else ""))


# because tuple is specifically special cased to go to ctuple where possible
def test_tuple(a: typing.Tuple[cython.int, cython.float], b: typing.Tuple[cython.int, ...],
               c: Tuple[cython.int, object]  # cannot be a ctuple
               ):
    """
    >>> test_tuple((1, 1.0), (1, 1.0), (1, 1.0))
    int
    int
    float
    Python object
    (int, float)
    tuple object
    tuple object
    tuple object
    tuple object
    """
    x: typing.Tuple[int, float] = (a[0], a[1])  # note: Python int/float, not cython.int/float
    y: Tuple[cython.int, ...] = (1,2.)
    plain_tuple: Tuple = ()
    z = a[0]  # should infer to C int
    p = x[1]  # should infer to Python float -> C double

    print(cython.typeof(z))
    print("int" if cython.compiled and cython.typeof(x[0]) == "Python object" else cython.typeof(x[0]))  # FIXME: infer Python int
    if cython.compiled:
        print(cython.typeof(p))
    else:
        print('float' if cython.typeof(p) == 'float' else cython.typeof(p))
    print(cython.typeof(x[1]) if cython.compiled or cython.typeof(p) != 'float' else "Python object")  # FIXME: infer C double
    print(cython.typeof(a) if cython.compiled or cython.typeof(a) != 'tuple' else "(int, float)")
    print(cython.typeof(x) + (" object" if not cython.compiled else ""))
    print(cython.typeof(y) + (" object" if not cython.compiled else ""))
    print(cython.typeof(c) + (" object" if not cython.compiled else ""))
    print(cython.typeof(plain_tuple) + (" object" if not cython.compiled else ""))


# because tuple is specifically special cased to go to ctuple where possible
def test_tuple_without_typing(a: tuple[cython.int, cython.float], b: tuple[cython.int, ...],
               c: tuple[cython.int, object]  # cannot be a ctuple
               ):
    """
    >>> test_tuple_without_typing((1, 1.0), (1, 1.0), (1, 1.0))
    int
    int
    float
    Python object
    (int, float)
    tuple object
    tuple object
    tuple object
    tuple object
    """
    x: tuple[int, float] = (a[0], a[1])  # note: Python int/float, not cython.int/float
    y: tuple[cython.int, ...] = (1,2.)
    plain_tuple: tuple = ()
    z = a[0]  # should infer to C int
    p = x[1]  # should infer to Python float -> C double

    print(cython.typeof(z))
    print("int" if cython.compiled and cython.typeof(x[0]) == "Python object" else cython.typeof(x[0]))  # FIXME: infer Python int
    print(cython.typeof(p) if cython.compiled or cython.typeof(p) != 'float' else "float")  # FIXME: infer C double/PyFloat from Py type
    print(cython.typeof(x[1]) if cython.compiled or cython.typeof(p) != 'float' else "Python object")  # FIXME: infer C double
    print(cython.typeof(a) if cython.compiled or cython.typeof(a) != 'tuple' else "(int, float)")
    print(cython.typeof(x) + (" object" if not cython.compiled else ""))
    print(cython.typeof(y) + (" object" if not cython.compiled else ""))
    print(cython.typeof(c) + (" object" if not cython.compiled else ""))
    print(cython.typeof(plain_tuple) + (" object" if not cython.compiled else ""))


_WARNINGS = """
"""



# cython: auto_cpdef=True
# mode:run
# tag: directive,auto_cpdef

import cython

def str(arg):
    """
    This is a bit evil - str gets mapped to a C-API function and is
    being redefined here.

    >>> print(str('TEST'))
    STR
    """
    return 'STR'

@cython.test_assert_path_exists('//SimpleCallNode[@function.type.is_cfunction = True]')
@cython.test_fail_if_path_exists('//SimpleCallNode[@function.type.is_builtin_type = True]')
def call_str(arg):
    """
    >>> print(call_str('TEST'))
    STR
    """
    return str(arg)


def stararg_func(*args):
    """
    >>> stararg_func(1, 2)
    (1, 2)
    """
    return args

def starstararg_func(**kwargs):
    """
    >>> starstararg_func(a=1)
    1
    """
    return kwargs['a']

l = lambda x: 1

def test_lambda():
    """
    >>> l(1)
    1
    """


# test classical import fallbacks
try:
    from math import fabs
except ImportError:
    def fabs(x):
        if x < 0:
            return -x
        else:
            return x

try:
    from math import no_such_function
except ImportError:
    def no_such_function(x):
        return x + 1.0


def test_import_fallback():
    """
    >>> fabs(1.0)
    1.0
    >>> no_such_function(1.0)
    2.0
    >>> test_import_fallback()
    (1.0, 2.0)
    """
    return fabs(1.0), no_such_function(1.0)



# mode: run
# tag: cpp, no-cpp-locals

cdef extern from *:
    """
    struct Foo
    {

      static const char* bar(int x, int y) {
        return "second";
      }

      static const char* bar(int x) {
        return "first";
      }

      const char* baz(int x, int y) {
        return "second";
      }

      const char* baz(int x) {
        return "first";
      }
    };
    """
    cppclass Foo:
        @staticmethod
        const char* bar(int x)

        @staticmethod
        const char* bar(int x, int y)

        const char* baz(int x)
        const char* baz(int x, int y)

def test_normal_method_overload():
    """
    >>> test_normal_method_overload()
    """
    cdef Foo f
    assert f.baz(1) == b"first"
    assert f.baz(1, 2) == b"second"

def test_static_method_overload():
    """
    >>> test_static_method_overload()
    """
    assert Foo.bar(1) == b"first"
    assert Foo.bar(1, 2) == b"second"



# mode: error
# tag: syntax

def inline func() -> int:
    pass

def api func() -> int:
    pass

def nogil func() -> int:
    pass

def func() nogil:
    pass

def inline int* func():
    pass


_ERRORS = u"""
4:11: Cannot use cdef modifier 'inline' in Python function signature. Use a decorator instead.
7:8: Cannot use cdef modifier 'api' in Python function signature. Use a decorator instead.
10:10: Cannot use cdef modifier 'nogil' in Python function signature. Use a decorator instead.
13:11: Cannot use cdef modifier 'nogil' in Python function signature. Use a decorator instead.
16:11: Cannot use cdef modifier 'inline' in Python function signature. Use a decorator instead.
16:14: Expected '(', found '*'. Did you use cdef syntax in a Python declaration? Use decorators and Python type annotations instead.
"""



# mode: error
# tag: werror

def simple_return():
    return
    print 'Where am I?'

def simple_loops(*args):
    for i in args:
        continue
        print 'Never be here'

    while True:
        break
        print 'Never be here'

def conditional(a, b):
    if a:
        return 1
    elif b:
        return 2
    else:
        return 3
    print 'oops'

def try_except():
    try:
        raise TypeError
    except ValueError:
        pass
    else:
        print 'unreachable'


_ERRORS = """
6:4: Unreachable code
11:8: Unreachable code
15:8: Unreachable code
24:4: Unreachable code
32:8: Unreachable code
"""



# mode: run

cimport cython

import sys

# The tests here are to do with a deterministic order of destructors which
# isn't reliable for PyPy. Therefore, on PyPy we treat the test as
# "compiles and doesn't crash"
IS_PYPY = hasattr(sys, 'pypy_version_info')

# Count number of times an object was deallocated twice. This should remain 0.
cdef int double_deallocations = 0
def assert_no_double_deallocations():
    if IS_PYPY:
        return
    global double_deallocations
    err = double_deallocations
    double_deallocations = 0
    assert not err


# Compute x = f(f(f(...(None)...))) nested n times and throw away the result.
# The real test happens when exiting this function: then a big recursive
# deallocation of x happens. We are testing two things in the tests below:
# that Python does not crash and that no double deallocation happens.
# See also https://github.com/python/cpython/pull/11841
def recursion_test(f, int n=2**20):
    x = None
    cdef int i
    for i in range(n):
        x = f(x)


@cython.trashcan(True)
cdef class Recurse:
    """
    >>> recursion_test(Recurse)
    >>> assert_no_double_deallocations()
    """
    cdef public attr
    cdef int deallocated

    def __cinit__(self, x):
        self.attr = x

    def __dealloc__(self):
        # Check that we're not being deallocated twice
        global double_deallocations
        double_deallocations += self.deallocated
        self.deallocated = 1


cdef class RecurseSub(Recurse):
    """
    >>> recursion_test(RecurseSub)
    >>> assert_no_double_deallocations()
    """
    cdef int subdeallocated

    def __dealloc__(self):
        # Check that we're not being deallocated twice
        global double_deallocations
        double_deallocations += self.subdeallocated
        self.subdeallocated = 1


@cython.freelist(4)
@cython.trashcan(True)
cdef class RecurseFreelist:
    """
    >>> recursion_test(RecurseFreelist)
    >>> recursion_test(RecurseFreelist, 1000)
    >>> assert_no_double_deallocations()
    """
    cdef public attr
    cdef int deallocated

    def __cinit__(self, x):
        self.attr = x

    def __dealloc__(self):
        # Check that we're not being deallocated twice
        global double_deallocations
        double_deallocations += self.deallocated
        self.deallocated = 1


# Subclass of list => uses trashcan by default
# As long as https://github.com/python/cpython/pull/11841 is not fixed,
# this does lead to double deallocations, so we skip that check.
cdef class RecurseList(list):
    """
    >>> RecurseList(42)
    [42]
    >>> recursion_test(RecurseList)
    """
    def __init__(self, x):
        super().__init__((x,))


# Some tests where the trashcan is NOT used. When the trashcan is not used
# in a big recursive deallocation, the __dealloc__s of the base classes are
# only run after the __dealloc__s of the subclasses.
# We use this to detect trashcan usage.
cdef int base_deallocated = 0
cdef int trashcan_used = 0
def assert_no_trashcan_used():
    if IS_PYPY:
        return
    global base_deallocated, trashcan_used
    err = trashcan_used
    trashcan_used = base_deallocated = 0
    assert not err


cdef class Base:
    def __dealloc__(self):
        global base_deallocated
        base_deallocated = 1


# Trashcan disabled by default
cdef class Sub1(Base):
    """
    >>> recursion_test(Sub1, 100)
    >>> assert_no_trashcan_used()
    """
    cdef public attr

    def __cinit__(self, x):
        self.attr = x

    def __dealloc__(self):
        global base_deallocated, trashcan_used
        trashcan_used += base_deallocated


@cython.trashcan(True)
cdef class Middle(Base):
    cdef public foo


# Trashcan disabled explicitly
@cython.trashcan(False)
cdef class Sub2(Middle):
    """
    >>> recursion_test(Sub2, 1000)
    >>> assert_no_trashcan_used()
    """
    cdef public attr

    def __cinit__(self, x):
        self.attr = x

    def __dealloc__(self):
        global base_deallocated, trashcan_used
        trashcan_used += base_deallocated



# mode: run
# tag: cpp, cpp11, no-cpp-locals

from libcpp.random cimport mt19937, mt19937_64, random_device, uniform_int_distribution, \
    uniform_real_distribution, bernoulli_distribution, binomial_distribution, \
    geometric_distribution, negative_binomial_distribution, poisson_distribution, \
    exponential_distribution, gamma_distribution, weibull_distribution, \
    extreme_value_distribution, normal_distribution, lognormal_distribution, \
    chi_squared_distribution, cauchy_distribution, fisher_f_distribution, student_t_distribution
from libc.float cimport DBL_MAX as DBL_MAX_


DBL_MAX = DBL_MAX_


def mt19937_seed_test():
    """
    >>> print(mt19937_seed_test())
    1608637542
    """
    cdef mt19937 gen = mt19937(42)
    return gen()


def mt19937_reseed_test():
    """
    >>> print(mt19937_reseed_test())
    1608637542
    """
    cdef mt19937 gen
    gen.seed(42)
    return gen()


def mt19937_min_max():
    """
    >>> x, y = mt19937_min_max()
    >>> print(x)
    0
    >>> print(y)  # 2 ** 32 - 1 because mt19937 is 32 bit.
    4294967295
    """
    cdef mt19937 gen
    return gen.min(), gen.max()


def mt19937_discard(z):
    """
    >>> x, y = mt19937_discard(13)
    >>> print(x)
    1972458954
    >>> print(y)
    1972458954
    """
    cdef mt19937 gen = mt19937(42)
    # Throw away z random numbers.
    gen.discard(z)
    a = gen()

    # Iterate over z random numbers.
    gen.seed(42)
    for _ in range(z + 1):
        b = gen()
    return a, b


def mt19937_64_seed_test():
    """
    >>> print(mt19937_64_seed_test())
    13930160852258120406
    """
    cdef mt19937_64 gen = mt19937_64(42)
    return gen()


def mt19937_64_reseed_test():
    """
    >>> print(mt19937_64_reseed_test())
    13930160852258120406
    """
    cdef mt19937_64 gen
    gen.seed(42)
    return gen()


def mt19937_64_min_max():
    """
    >>> x, y = mt19937_64_min_max()
    >>> print(x)
    0
    >>> print(y)  # 2 ** 64 - 1 because mt19937_64 is 64 bit.
    18446744073709551615
    """
    cdef mt19937_64 gen
    return gen.min(), gen.max()


def mt19937_64_discard(z):
    """
    >>> x, y = mt19937_64_discard(13)
    >>> print(x)
    11756813601242511406
    >>> print(y)
    11756813601242511406
    """
    cdef mt19937_64 gen = mt19937_64(42)
    # Throw away z random numbers.
    gen.discard(z)
    a = gen()

    # Iterate over z random numbers.
    gen.seed(42)
    for _ in range(z + 1):
        b = gen()
    return a, b


ctypedef fused any_dist:
    uniform_int_distribution[int]
    uniform_real_distribution[double]
    bernoulli_distribution
    binomial_distribution[int]
    geometric_distribution[int]
    negative_binomial_distribution[int]
    poisson_distribution[int]
    exponential_distribution[double]
    gamma_distribution[double]
    weibull_distribution[double]
    extreme_value_distribution[double]
    normal_distribution[double]
    lognormal_distribution[double]
    chi_squared_distribution[double]
    cauchy_distribution[double]
    fisher_f_distribution[double]
    student_t_distribution[double]


cdef sample_or_range(any_dist dist, bint sample):
    """
    This helper function returns a sample if `sample` is truthy and the range of the distribution
    if `sample` is falsy. We use a fused type to avoid duplicating the conditional statement in each
    distribution test.
    """
    cdef random_device rd
    if sample:
        dist(mt19937(rd()))
    else:
        return dist.min(), dist.max()


def uniform_int_distribution_test(a, b, sample=True):
    """
    >>> uniform_int_distribution_test(2, 3)
    >>> uniform_int_distribution_test(5, 9, False)
    (5, 9)
    """
    cdef uniform_int_distribution[int] dist = uniform_int_distribution[int](a, b)
    return sample_or_range[uniform_int_distribution[int]](dist, sample)


def uniform_real_distribution_test(a, b, sample=True):
    """
    >>> x = uniform_real_distribution_test(4, 5)
    >>> uniform_real_distribution_test(3, 8, False)
    (3.0, 8.0)
    """
    cdef uniform_real_distribution[double] dist = uniform_real_distribution[double](a, b)
    return sample_or_range[uniform_real_distribution[double]](dist, sample)


def bernoulli_distribution_test(proba, sample=True):
    """
    >>> bernoulli_distribution_test(0.2)
    >>> bernoulli_distribution_test(0.7, False)
    (False, True)
    """
    cdef bernoulli_distribution dist = bernoulli_distribution(proba)
    return sample_or_range[bernoulli_distribution](dist, sample)


def binomial_distribution_test(n, proba, sample=True):
    """
    >>> binomial_distribution_test(10, 0.7)
    >>> binomial_distribution_test(75, 0.3, False)
    (0, 75)
    """
    cdef binomial_distribution[int] dist = binomial_distribution[int](n, proba)
    return sample_or_range[binomial_distribution[int]](dist, sample)


def geometric_distribution_test(proba, sample=True):
    """
    >>> geometric_distribution_test(.4)
    >>> geometric_distribution_test(0.2, False)  # 2147483647 = 2 ** 32 - 1
    (0, 2147483647)
    """
    cdef geometric_distribution[int] dist = geometric_distribution[int](proba)
    return sample_or_range[geometric_distribution[int]](dist, sample)


def negative_binomial_distribution_test(n, p, sample=True):
    """
    >>> negative_binomial_distribution_test(5, .1)
    >>> negative_binomial_distribution_test(10, 0.2, False)  # 2147483647 = 2 ** 32 - 1
    (0, 2147483647)
    """
    cdef negative_binomial_distribution[int] dist = negative_binomial_distribution[int](n, p)
    return sample_or_range[negative_binomial_distribution[int]](dist, sample)


def poisson_distribution_test(rate, sample=True):
    """
    >>> poisson_distribution_test(7)
    >>> poisson_distribution_test(7, False)  # 2147483647 = 2 ** 32 - 1
    (0, 2147483647)
    """
    cdef poisson_distribution[int] dist = poisson_distribution[int](rate)
    return sample_or_range[poisson_distribution[int]](dist, sample)


def exponential_distribution_test(rate, sample=True):
    """
    >>> x = exponential_distribution_test(6)
    >>> l, u = exponential_distribution_test(1, False)
    >>> l
    0.0
    >>> u == DBL_MAX or u == float("inf")
    True
    """
    cdef exponential_distribution[double] dist = exponential_distribution[double](rate)
    return sample_or_range[exponential_distribution[double]](dist, sample)


def gamma_distribution_test(shape, scale, sample=True):
    """
    >>> gamma_distribution_test(3, 4)
    >>> l, u = gamma_distribution_test(1, 1, False)
    >>> l
    0.0
    >>> u == DBL_MAX or u == float("inf")
    True
    """
    cdef gamma_distribution[double] dist = gamma_distribution[double](shape, scale)
    return sample_or_range[gamma_distribution[double]](dist, sample)


def weibull_distribution_test(shape, scale, sample=True):
    """
    >>> weibull_distribution_test(3, 2)
    >>> l, u = weibull_distribution_test(1, 1, False)
    >>> l
    0.0
    >>> u == DBL_MAX or u == float("inf")
    True
    """
    cdef weibull_distribution[double] dist = weibull_distribution[double](shape, scale)
    return sample_or_range[weibull_distribution[double]](dist, sample)


def extreme_value_distribution_test(shape, scale, sample=True):
    """
    >>> extreme_value_distribution_test(3, 0.1)
    >>> l, u = extreme_value_distribution_test(1, 1, False)
    >>> l == -DBL_MAX or l == -float("inf")
    True
    >>> u == DBL_MAX or u == float("inf")
    True
    """
    cdef extreme_value_distribution[double] dist = extreme_value_distribution[double](shape, scale)
    return sample_or_range[extreme_value_distribution[double]](dist, sample)


def normal_distribution_test(loc, scale, sample=True):
    """
    >>> normal_distribution_test(3, 2)
    >>> l, u = normal_distribution_test(1, 1, False)
    >>> l == -DBL_MAX or l == -float("inf")
    True
    >>> u == DBL_MAX or u == float("inf")
    True
    """
    cdef normal_distribution[double] dist = normal_distribution[double](loc, scale)
    return sample_or_range[normal_distribution[double]](dist, sample)


def lognormal_distribution_test(loc, scale, sample=True):
    """
    >>> lognormal_distribution_test(3, 2)
    >>> l, u = lognormal_distribution_test(1, 1, False)
    >>> l
    0.0
    >>> u == DBL_MAX or u == float("inf")
    True
    """
    cdef lognormal_distribution[double] dist = lognormal_distribution[double](loc, scale)
    return sample_or_range[lognormal_distribution[double]](dist, sample)


def chi_squared_distribution_test(dof, sample=True):
    """
    >>> x = chi_squared_distribution_test(9)
    >>> l, u = chi_squared_distribution_test(5, False)
    >>> l
    0.0
    >>> u == DBL_MAX or u == float("inf")
    True
    """
    cdef chi_squared_distribution[double] dist = chi_squared_distribution[double](dof)
    return sample_or_range[chi_squared_distribution[double]](dist, sample)


def cauchy_distribution_test(loc, scale, sample=True):
    """
    >>> cauchy_distribution_test(3, 9)
    >>> l, u = cauchy_distribution_test(1, 1, False)
    >>> l == -DBL_MAX or l == -float("inf")
    True
    >>> u == DBL_MAX or u == float("inf")
    True
    """
    cdef cauchy_distribution[double] dist = cauchy_distribution[double](loc, scale)
    return sample_or_range[cauchy_distribution[double]](dist, sample)


def fisher_f_distribution_test(m, n, sample=True):
    """
    >>> x = fisher_f_distribution_test(9, 11)
    >>> l, u = fisher_f_distribution_test(1, 1, False)
    >>> l
    0.0
    >>> u == DBL_MAX or u == float("inf")
    True
    """
    cdef fisher_f_distribution[double] dist = fisher_f_distribution[double](m, n)
    return sample_or_range[fisher_f_distribution[double]](dist, sample)


def student_t_distribution_test(dof, sample=True):
    """
    >>> x = student_t_distribution_test(13)
    >>> l, u = student_t_distribution_test(1, False)
    >>> l == -DBL_MAX or l == -float("inf")
    True
    >>> u == DBL_MAX or u == float("inf")
    True
    """
    cdef student_t_distribution[double] dist = student_t_distribution[double](dof)
    return sample_or_range[student_t_distribution[double]](dist, sample)



# ticket: t183

cimport cython

@cython.cdivision(True)
cpdef cdiv_decorator(int a, int b):
    """
    >>> cdiv_decorator(-12, 5)
    -2
    """
    return a / b

@cython.cdivision(False)
cpdef pydiv_decorator(int a, int b):
    """
    >>> pydiv_decorator(-12, 5)
    -3
    """
    return a / b



# mode: run

import sys

def test(obj):
    """
    >>> test(Exception('hi'))
    Raising: Exception('hi',)
    Caught: Exception('hi',)
    """
    print u"Raising: %s%r" % (obj.__class__.__name__, obj.args)
    try:
        raise obj
    except:
        info = sys.exc_info()
        assert isinstance(info[0], type)
        print u"Caught: %s%r" % (info[1].__class__.__name__, info[1].args)



# mode: compile

cdef class Grail:

    def __add__(int x, float y):
        pass

cdef class Swallow:
    pass

def f(Grail g):
    cdef int i = 0
    cdef Swallow s = Swallow()
    cdef object x = Grail()
    g = x
    x = g
    g = i
    i = g
    g = s
    s = g




# cython: language_level=2, warn.maybe_uninitialized=True
# mode: error
# tag: werror

def list_comp(a):
    r = [i for i in a]
    return i

# dict comp is py3 feuture and don't leak here
def dict_comp(a):
    r = {i: j for i, j in a}
    return i, j

def dict_comp2(a):
    r = {i: j for i, j in a}
    print i, j
    i, j = 0, 0


_ERRORS = """
7:11: local variable 'i' might be referenced before assignment
12:11: undeclared name not builtin: i
12:14: undeclared name not builtin: j
16:10: local variable 'i' referenced before assignment
16:13: local variable 'j' referenced before assignment
"""



# mode: compile
# tag: gh6860

# Make sure we generate the C complex type declarations
# even if the user code does not reference the type explicitly.

def c(x, y):
    return complex(x) / complex(y)



# mode: error

cimport cython

def closure(cython.integral i):
    def inner(cython.floating f):
        pass

def closure2(cython.integral i):
    return lambda cython.integral i: i

def closure3(cython.integral i):
    def inner():
        return lambda cython.floating f: f

def generator(cython.integral i):
    yield i

_ERRORS = u"""
e_fused_closure.pyx:6:4: Cannot nest fused functions
e_fused_closure.pyx:10:11: Fused lambdas not allowed
e_fused_closure.pyx:14:15: Fused lambdas not allowed
e_fused_closure.pyx:16:0: Fused generators not supported
"""



# tag: cpp

import math
cimport libcpp.complex as cppcomplex
from libcpp.complex cimport complex as complex_class

def double_complex(complex_class[double] a):
    """
    >>> double_complex(1 + 2j)
    (1+2j)
    >>> double_complex(1.5 + 2.5j)
    (1.5+2.5j)
    """
    return a

def double_int(complex_class[int] a):
    """
    >>> double_int(1 + 2j)
    (1+2j)
    >>> double_int(1.5 + 2.5j)
    (1+2j)
    """
    return a

def double_addition_with_scalar(complex_class[double] a, double b):
    """
    >>> double_addition_with_scalar(1 + 1j, 1) + 1
    (3+1j)
    >>> 1 + double_addition_with_scalar(1 + 2j, -1)
    (1+2j)
    >>> 1j + double_addition_with_scalar(1 + 2j, 7)
    (8+3j)
    >>> double_addition_with_scalar(1 + 2j, 7) - 1j
    (8+1j)
    >>> double_addition_with_scalar(1.5 + 2.5j, -1) + 0
    (0.5+2.5j)
    >>> -0 + double_addition_with_scalar(1.5 + 2.5j, -1)
    (0.5+2.5j)
    """
    cdef complex_class[double] c = a + b
    cdef complex_class[double] d = b + a
    return b + a

def double_inplace_addition_with_scalar(complex_class[double] a, double b):
    """
    >>> double_inplace_addition_with_scalar(1 + 1j, 1)
    (2+1j)
    """
    a += b
    return a

def double_inplace_substraction_with_scalar(complex_class[double] a, double b):
    """
    >>> double_inplace_substraction_with_scalar(1 + 1j, 2)
    (-1+1j)
    """
    a -= b
    return a

def double_inplace_multiplication_with_scalar(complex_class[double] a, double b):
    """
    >>> double_inplace_multiplication_with_scalar(1 + 1j, 2)
    (2+2j)
    """
    a *= b
    return a

def double_inplace_division_with_scalar(complex_class[double] a, double b):
    """
    >>> double_inplace_division_with_scalar(3 + 3j, 2)
    (1.5+1.5j)
    """
    a /= b
    return a

def double_unary_negation(complex_class[double] a):
    """
    >>> double_unary_negation(2 + 2j)
    (-2-2j)
    """
    return -a

def double_unary_positive(complex_class[double] a):
    """
    >>> double_unary_positive(2 + 2j)
    (2+2j)
    """
    return +a

def double_real_imaginary_accessors(complex_class[double] a, double real, double imag):
    """
    >>> a = double_real_imaginary_accessors(2.1 + 2j, 7, 4)
    >>> a.real
    9.1
    >>> a.imag
    6.0
    """
    # checks the accessors
    cdef double b = a.real()
    a.real(b + real)

    cdef double c = a.imag()
    a.imag(c + imag)

    return a

def double_double_comparison_equal(complex_class[double] a, complex_class[double] b):
    """
    >>> double_double_comparison_equal(2.1 + 2j, 7)
    False
    >>> double_double_comparison_equal(2.1 + 2j, (1j + 1) * 2 + 0.1)
    True
    """
    return a == b

def double_scalar_double_comparison_equal(complex_class[double] a, double b):
    """
    >>> double_double_comparison_equal(2.1 + 2j, 7)
    False
    >>> double_double_comparison_equal(7 + 0j, 7)
    True
    """
    return a == b

def scalar_double_double_comparison_equal(complex_class[double] a, double b):
    """
    >>> double_double_comparison_equal(2.1 + 2j, 7)
    False
    >>> double_double_comparison_equal(7 + 0j, 7)
    True
    """
    return b == a

def double_real_imaginary_accessors_free_function(complex_class[double] a, bint real_part):
    """
    >>> double_real_imaginary_accessors_free_function(2.1 + 2.7j, True)
    2.1
    >>> double_real_imaginary_accessors_free_function(2.1 + 2.7j, False)
    2.7
    """

    cdef double e = cppcomplex.real(a)
    cdef double f = cppcomplex.imag(a)
    if real_part:
        return e
    return f

def scalar_double_real_imaginary_accessors_free_function(double a, bint real_part):
    """
    >>> scalar_double_real_imaginary_accessors_free_function(2.1, True)
    2.1
    >>> scalar_double_real_imaginary_accessors_free_function(2.1, False)
    0.0
    """
    cdef double e = cppcomplex.real(a)
    cdef double f = cppcomplex.imag(a)
    if real_part:
        return e
    return f

def scalar_long_double_real_imaginary_accessors_free_function(long double a, bint real_part):
    """
    >>> scalar_long_double_real_imaginary_accessors_free_function(2.1, True)
    2.1
    >>> scalar_long_double_real_imaginary_accessors_free_function(2.1, False)
    0.0
    """
    cdef double e = cppcomplex.real(a)
    cdef double f = cppcomplex.imag(a)
    if real_part:
        return e
    return f

def double_abs(complex_class[double] a):
    """
    >>> double_abs(5)
    5.0
    >>> double_abs(5j)
    5.0
    >>> double_abs(2 + 5j) == math.sqrt(29)
    True
    """
    return cppcomplex.abs(a)

def double_norm(complex_class[double] a):
    """
    >>> double_norm(5)
    25.0
    >>> double_norm(5j)
    25.0
    >>> abs(double_norm(2 + 5j) - double_abs(2 + 5j)*double_abs(2 + 5j)) < 1e-13
    True
    """
    return cppcomplex.norm(a)

def scalar_double_norm(double a):
    """
    >>> scalar_double_norm(5)
    25.0
    """
    return cppcomplex.norm(a)

def scalar_float_norm(float a):
    """
    >>> scalar_float_norm(5)
    25.0
    """
    return cppcomplex.norm(a)

def double_conjugate(complex_class[double] a):
    """
    >>> double_conjugate(5)
    (5-0j)
    >>> double_conjugate(5j)
    -5j
    >>> double_conjugate(1 + 2j)
    (1-2j)
    """
    return cppcomplex.conj(a)

def scalar_double_conjugate(double a):
    """
    >>> a = scalar_double_conjugate(5)
    >>> a.real
    5.0
    >>> # abs to prevent -0 or 0 issue
    >>> abs(a.imag)
    0.0
    """
    # always return complex
    return cppcomplex.conj(a)

def double_proj(complex_class[double] a):
    """
    >>> double_proj(5 + 4j)
    (5+4j)
    >>> double_proj(-float("infinity") + 4j)
    (inf+0j)
    >>> double_proj(5 - float("infinity")*1j)
    (inf-0j)
    """
    return cppcomplex.proj(a)

def double_arg(complex_class[double] a):
    """
    >>> a = math.pi / 4
    >>> round(a, 10)
    0.7853981634
    >>> round(double_arg(math.cos(a) + math.sin(a)*1j), 10)
    0.7853981634
    """
    return cppcomplex.arg(a)

def scalar_double_arg(double a):
    """
    >>> scalar_double_arg(13)
    0.0
    >>> round(scalar_double_arg(-1), 10)
    3.1415926536
    """
    return cppcomplex.arg(a)

def double_polar(double r, double theta):
    """
    >>> c1 = double_polar(3, math.pi / 2)
    >>> c1.imag
    3.0
    >>> abs(c1.real) < 1e-10
    True
    >>> c2 = double_polar(4, math.pi)
    >>> c2.real
    -4.0
    >>> abs(c2.imag) < 1e-10
    True
    """
    return cppcomplex.polar(r, theta)

def double_polar_scalar(double r):
    """
    >>> c1 = double_polar_scalar(3)
    >>> c1.real
    3.0
    >>> c1.imag
    0.0
    >>> c2 = double_polar_scalar(0)
    >>> c2.real
    0.0
    >>> c2.imag
    0.0
    """
    return cppcomplex.polar(r)

def double_pow(complex_class[double] a, complex_class[double] b):
    """
    >>> double_pow(3, 3)
    (27+0j)
    >>> a = double_pow(1j, 1j)
    >>> round(a.real, 5)
    0.20788
    >>> round(a.imag, 2)
    0.0
    """
    return cppcomplex.pow(a, b)

def double_scalar_double_pow(complex_class[double] a, double b):
    """
    >>> double_pow(3, 3)
    (27+0j)
    >>> a = double_pow(1+2j, 2)
    >>> round(a.real, 2)
    -3.0
    >>> round(a.imag, 2)
    4.0
    """
    return cppcomplex.pow(a, b)

def scalar_double_double_pow(double a, complex_class[double] b):
    """
    >>> scalar_double_double_pow(3, 3)
    (27+0j)
    >>> a = scalar_double_double_pow(2, 2j)
    >>> round(a.real, 2)
    0.18
    >>> round(a.imag, 2)
    0.98
    """
    return cppcomplex.pow(a, b)

def double_sin(complex_class[double] a):
    """
    >>> round(abs(double_sin(math.pi)), 2)
    0.0
    >>> round(double_sin(-math.pi/4).real, 3)
    -0.707
    >>> round(abs(double_sin(-math.pi/4).imag), 3)
    0.0
    >>> round(abs(double_sin(4j).real), 3)
    0.0
    >>> round(abs(double_sin(4j).imag - math.sinh(4)), 3)
    0.0
    """
    return cppcomplex.sin(a)

def double_cos(complex_class[double] a):
    """
    >>> round(double_cos(math.pi).real, 2)
    -1.0
    >>> round(abs(double_cos(math.pi).imag), 2)
    0.0
    >>> round(double_cos(-math.pi/4 + math.pi).real, 3)
    -0.707
    >>> round(abs(double_cos(-math.pi/4).imag), 3)
    0.0
    >>> round(abs(double_cos(4j).imag), 3)
    0.0
    >>> round(abs(double_cos(5j).real - math.cosh(5)), 3)
    0.0
    """
    return cppcomplex.cos(a)



"""
Initial cleanup and 'calibration':
>>> _ = gc.collect()
>>> old_unreachable = len(gc.garbage)

Test:
>>> x = SimpleGarbage()
SimpleGarbage(1) __cinit__
>>> del x
SimpleGarbage(1) __dealloc__
Collector.__dealloc__

Make sure nothing changed in the environment:
>>> new_unreachable = get_new_unreachable()
>>> new_unreachable == old_unreachable or (old_unreachable, new_unreachable)
True
"""

import gc

cdef Py_ssize_t new_unreachable = 0

def get_new_unreachable():
    return new_unreachable

cdef int counter = 0
cdef int next_counter():
    global counter
    counter += 1
    return counter

cdef class Collector:
    # Indirectly trigger garbage collection in SimpleGarbage deallocation.
    # The __dealloc__ method of SimpleGarbage won't trigger the bug as the
    # refcount is artificially inflated for the duration of that function.
    def __dealloc__(self):
        print "Collector.__dealloc__"
        global new_unreachable
        gc.collect()
        new_unreachable = len(gc.garbage)

cdef class SimpleGarbage:
    cdef Collector c  # to participate in garbage collection
    cdef int index
    cdef bint deallocated
    def __cinit__(self):
        self.index = next_counter()
        self.c = Collector()
        print self, "__cinit__"
    def __dealloc__(self):
        print self, "__dealloc__"
        if self.deallocated:
            print "Double dealloc!"
        self.deallocated = True
        gc.collect()
    def __str__(self):
        return "SimpleGarbage(%s)" % self.index
    def __repr__(self):
        return "SimpleGarbage(%s)" % self.index



# mode: run
# tag: cpp20, cpp

from libcpp.numbers cimport pi, e


def test_pi():
    """
    >>> import math
    >>> test_pi() == math.pi
    True
    """
    return pi

def test_e():
    """
    >>> import math
    >>> test_e() == math.e
    True
    """
    return e



# ticket: 3293
# mode: run
# tag: cpp, cpp17, no-cpp-locals, werror

from cython.operator cimport dereference as deref
from libcpp.optional cimport optional, nullopt, make_optional
from libcpp.string cimport string
from libcpp.pair cimport pair

def simple_test():
    """
    >>> simple_test()
    """
    cdef optional[int] o
    assert(not o.has_value())
    try:
        o.value()
    except Exception as err:
        pass
    else:
        assert False, "value() did not raise a catchable error"
    o = 5
    assert(o.has_value())
    assert(o.value()==5)
    o.reset()
    assert(not o.has_value())

def operator_test():
    """
    >>> operator_test()
    """
    cdef optional[int] o1, o2

    # operator bool
    assert(not o1)
    o1 = 5
    assert(o1)

    # operator *
    assert(deref(o1) == 5)

    # operator =,==,!=,>,<,>=,<=
    assert(not o1 == o2)
    assert(o1 != o2)
    o2 = o1
    assert(o1 == o2)
    assert(not o1 > o2)
    assert(not o1 < o2)
    assert(o1 >= o2)
    assert(o1 <= o2)

    # operators =,== for other types (all related operators are identical)
    o1 = 6
    assert(o1 == 6)
    o2 = nullopt
    assert(o2 == nullopt)

def misc_methods_test():
    """
    >>> misc_methods_test()
    """

    # make_optional
    cdef optional[int] o
    o = make_optional[int](5)
    assert(o == 5)

    # swap
    o.swap(optional[int](6))
    assert(o == 6)

    # emplace
    cdef optional[pair[int,int]] op
    cdef pair[int,int]* val_ptr = &op.emplace(1,2)
    assert(op.has_value())
    assert(op.value() == pair[int,int](1,2))
    assert(&op.value() == val_ptr) # check returned reference
    



 
from libc.stdlib cimport free


cdef class OwnedPointer:
    cdef void* ptr

    def __dealloc__(self):
        if self.ptr is not NULL:
            free(self.ptr)


    @staticmethod
    cdef create(void* ptr):
        p = OwnedPointer()
        p.ptr = ptr
        return p



##################### UFuncDefinition ######################

cdef extern from *:
    ctypedef int npy_intp
    struct PyObject
    PyObject* __Pyx_NewRef(object)

# variable names have to come from tempita to avoid duplication
@cname("{{func_cname}}")
cdef void {{func_cname}}(char **args, const npy_intp *dimensions, const npy_intp* steps, void* data) except * {{"nogil" if will_be_called_without_gil else ""}}:
    cdef npy_intp i
    cdef npy_intp n = dimensions[0]
    {{for idx, tn_tp in enumerate(in_types)}}
    cdef char* in_{{idx}} = args[{{idx}}]
    cdef {{tn_tp[0]}} cast_in_{{idx}}
    {{endfor}}
    {{for idx, tn_tp in enumerate(out_types)}}
    cdef char* out_{{idx}} = args[{{idx+len(in_types)}}]
    cdef {{tn_tp[0]}} cast_out_{{idx}}
    {{endfor}}
    {{for idx in range(len(out_types)+len(in_types))}}
    cdef npy_intp step_{{idx}} = steps[{{idx}}]
    {{endfor}}

    {{"with gil" if (not nogil and will_be_called_without_gil) else "if True"}}:
        for i in range(n):
            {{for idx, tn_tp in enumerate(in_types)}}
            {{if tn_tp[1].is_pyobject}}
            cast_in_{{idx}} = (<{{tn_tp[0]}}>(<void**>in_{{idx}})[0])
            {{else}}
            cast_in_{{idx}} = (<{{tn_tp[0]}}*>in_{{idx}})[0]
            {{endif}}
            {{endfor}}

            {{", ".join("cast_out_{}".format(idx) for idx in range(len(out_types)))}} = \
                {{inline_func_call}}({{", ".join("cast_in_{}".format(idx) for idx in range(len(in_types)))}})

            {{for idx, tn_tp in enumerate(out_types)}}
            {{if tn_tp[1].is_pyobject}}
            (<void**>out_{{idx}})[0] = <void*>__Pyx_NewRef(cast_out_{{idx}})
            {{else}}
            (<{{tn_tp[0]}}*>out_{{idx}})[0] = cast_out_{{idx}}
            {{endif}}
            {{endfor}}
            {{for idx in range(len(in_types))}}
            in_{{idx}} += step_{{idx}}
            {{endfor}}
            {{for idx in range(len(out_types))}}
            out_{{idx}} += step_{{idx+len(in_types)}}
            {{endfor}}



# mode: error

cdef class C:
    cdef void f(self):
        pass

cdef class D(C):
    cdef void f(self, int x):
        pass

# These are declared in the pxd.
cdef class Base(object):
  cdef f(self):
    pass

cdef class MissingRedeclaration(Base):
  # Not declared (so assumed cdef) in the pxd.
  cpdef f(self):
    pass

cdef class BadRedeclaration(Base):
  # Declared as cdef in the pxd.
  cpdef f(self):
    pass

cdef class UnneededRedeclaration(Base):
  # This is OK, as it's not declared in the pxd.
  cpdef f(self):
    pass

cdef class NarrowerReturn(Base):
  # This does not require a new vtable entry.
  cdef Base f(self):
    pass


_ERRORS = u"""
8: 4: Signature not compatible with previous declaration
4: 4: Previous declaration is here
# TODO(robertwb): Re-enable these errors.
#18:4: Compatible but non-identical C method 'f' not redeclared in definition part of extension type 'MissingRedeclaration'
#2:4: Previous declaration is here
#23:4: Compatible but non-identical C method 'f' not redeclared in definition part of extension type 'BadRedeclaration'
#2:4: Previous declaration is here
"""



# tag: cpp

cimport cython
from libcpp.vector cimport vector
from libcpp.map cimport map
from libcpp.typeinfo cimport type_info
from cython.operator cimport typeid

def test_cpp_specialization(cython.floating element):
    """
    >>> import cython
    >>> test_cpp_specialization[cython.float](10.0)
    vector[float] * float 10.0
    >>> test_cpp_specialization[cython.double](10.0)
    vector[double] * double 10.0
    """
    cdef vector[cython.floating] *v = new vector[cython.floating]()
    v.push_back(element)
    print cython.typeof(v), cython.typeof(element), v.at(0)

cdef fused C:
   int
   object

cdef const type_info* tidint = &typeid(int)
def typeid_call(C x):
    """
    For GH issue 3203
    >>> typeid_call(1)
    True
    """
    cdef const type_info* a = &typeid(C)
    return a[0] == tidint[0]

cimport cython

def typeid_call2(cython.integral x):
    """
    For GH issue 3203
    >>> typeid_call2[int](1)
    True
    """
    cdef const type_info* a = &typeid(cython.integral)
    return a[0] == tidint[0]

cdef fused_ref(cython.integral& x):
    return x*2

def test_fused_ref(int x):
    """
    >>> test_fused_ref(5)
    (10, 10)
    """
    return fused_ref(x), fused_ref[int](x)

ctypedef fused nested_fused:
    vector[cython.integral]

cdef vec_of_fused(nested_fused v):
    x = v[0]
    return cython.typeof(x)

def test_nested_fused():
    """
    >>> test_nested_fused()
    int
    long
    """
    cdef vector[int] vi = [0,1]
    cdef vector[long] vl = [0,1]
    print vec_of_fused(vi)
    print vec_of_fused(vl)

ctypedef fused nested_fused2:
    map[cython.integral, cython.floating]

cdef map_of_fused(nested_fused2 m):
    for pair in m:
        return cython.typeof(pair.first), cython.typeof(pair.second)

def test_nested_fused2():
    """
    >>> test_nested_fused2()
    ('int', 'float')
    ('long', 'double')
    """
    cdef map[int, float] mif = { 0: 0.0 }
    cdef map[long, double] mld = { 0: 0.0 }
    print map_of_fused(mif)
    print map_of_fused(mld)



# mode: run


cdef int grail():
    cdef int (*spam)()
    spam = &grail
    spam = grail
    assert spam is grail
    assert spam == grail
    assert spam == &grail


ctypedef int funcptr_t()

cdef funcptr_t* get_grail():
    return &grail


def test_assignments():
    """
    >>> test_assignments()
    """
    grail()


def test_return_value():
    """
    >>> test_return_value()
    True
    """
    g = get_grail()
    return g == &grail


def call_cfuncptr():
    """
    >>> call_cfuncptr()
    """
    cdef int (*spam)()
    spam = grail
    spam()

cdef int exceptminus2(int bad) except -2:
    if bad:
        raise RuntimeError
    else:
        return 0

def call_exceptminus2_through_exceptstar_pointer(bad):
    """
    >>> call_exceptminus2_through_exceptstar_pointer(True)
    Traceback (most recent call last):
    ...
    RuntimeError
    >>> call_exceptminus2_through_exceptstar_pointer(False)
    0
    """
    cdef int (*fptr)(int) except *  # GH4770 - should not be treated as except? -1
    fptr = exceptminus2
    return fptr(bad)

def call_exceptminus2_through_exceptmaybeminus2_pointer(bad):
    """
    >>> call_exceptminus2_through_exceptmaybeminus2_pointer(True)
    Traceback (most recent call last):
    ...
    RuntimeError
    >>> call_exceptminus2_through_exceptmaybeminus2_pointer(False)
    0
    """
    cdef int (*fptr)(int) except ?-2  # exceptions should be compatible
    fptr = exceptminus2
    return fptr(bad)

cdef int noexcept_func():  # noexcept
    return 0

def call_noexcept_func_except_star():
    """
    >>> call_noexcept_func_except_star()
    0
    """
    cdef int (*fptr)() except *
    fptr = noexcept_func  # exception specifications are compatible
    return fptr()

def call_noexcept_func_except_check():
    """
    >>> call_noexcept_func_except_check()
    0
    """
    cdef int (*fptr)() except ?-1
    fptr = noexcept_func  # exception specifications are compatible
    return fptr()



# mode: compile

# Complex numbers defined in annotations weren't having their utility code imported directly
# leading to compile-errors that the type wasn't defined. The test is intentionally minimal since
# anything more thorough ends up creating the utility code
cdef f(x: complex):
    pass



# tag: cpp
# mode: compile

from libcpp.vector cimport vector

cdef extern from *:
    cdef cppclass Foo[T]:
        pass

    cdef cppclass Bar:
        pass

cdef vector[vector[int]] a
cdef vector[vector[const int]] b
cdef vector[vector[vector[int]]] c
cdef vector[vector[vector[const int]]] d
cdef Foo[Foo[Bar]] e
cdef Foo[Foo[const Bar]] f



# cython: language_level=3, binding=True, annotation_typing=False
# mode: run
# tag: generators, python3, exceptions, gh2230, gh2811

print(end='')  # test that language_level 3 applies immediately at the module start, for the first token.

cimport cython

__doc__ = """
>>> items = sorted(locals_function(1).items())
>>> for item in items:
...     print('%s = %r' % item)
a = 1
b = 2
x = 'abc'

>>> except_as_deletes
True

>>> no_match_does_not_touch_target
True
"""


def locals_function(a, b=2):
    x = 'abc'
    return locals()


### "new style" classes

class T:
    """
    >>> t = T()
    >>> isinstance(t, T)
    True
    >>> isinstance(T, type)  # not a Py2 old style class!
    True
    """


### true division

def truediv(x):
    """
    >>> truediv(4)
    2.0
    >>> truediv(3)
    1.5
    """
    return x / 2


def truediv_int(int x):
    """
    >>> truediv_int(4)
    2.0
    >>> truediv_int(3)
    1.5
    """
    return x / 2


@cython.cdivision(True)
def cdiv_int(int x):
    """
    >>> cdiv_int(4)
    2
    >>> cdiv_int(3)
    1
    """
    return x / 2


### module level except-as tests

exc = [None]
e = None
try:
    raise KeyError
except AttributeError as e:
    exc[0] = e
except KeyError       as e:
    exc[0] = e
except IndexError     as e:
    exc[0] = e
except:
    exc[0] = 'SOMETHING ELSE'

try:
    e
except NameError:
    except_as_deletes = True
else:
    except_as_deletes = False


e = 123
try:
    raise TypeError
except NameError as e:
    pass
except TypeError:
    pass
no_match_does_not_touch_target = (e == 123)


### more except-as tests

def except_as_no_raise_does_not_touch_target(a):
    """
    >>> except_as_no_raise_does_not_touch_target(TypeError)
    (1, 1)
    """
    d = a  # mark used

    b = 1
    try:
        i = 1
    except a as b:
        i = 2
    return i, b


def except_as_raise_deletes_target(x, a):
    """
    >>> except_as_raise_deletes_target(None, TypeError)
    1
    1
    >>> except_as_raise_deletes_target(TypeError('test'), TypeError)
    Traceback (most recent call last):
    UnboundLocalError: local variable 'b' referenced before assignment
    >>> except_as_raise_deletes_target(ValueError('test'), TypeError)
    Traceback (most recent call last):
    ValueError: test
    >>> except_as_raise_deletes_target(None, TypeError)
    1
    1
    """
    b = 1
    try:
        i = 1
        if x:
            raise x
    except a as b:
        i = 2
        assert isinstance(b, a)
    print(b)  # raises UnboundLocalError if except clause was executed
    return i


def except_as_raise_deletes_target_even_after_del(x, a):
    """
    >>> except_as_raise_deletes_target_even_after_del(None, TypeError)
    1
    1
    >>> except_as_raise_deletes_target_even_after_del(TypeError('test'), TypeError)
    2
    >>> except_as_raise_deletes_target_even_after_del(ValueError('test'), TypeError)
    Traceback (most recent call last):
    ValueError: test
    >>> except_as_raise_deletes_target_even_after_del(None, TypeError)
    1
    1
    """
    b = 1
    try:
        i = 1
        if x:
            raise x
    except a as b:
        i = 2
        assert isinstance(b, a)
        del b  # let's see if Cython can still 'del' it after this line!
    try:
        print(b)  # raises UnboundLocalError if except clause was executed
    except UnboundLocalError:
        pass
    else:
        if x:
            print("UnboundLocalError not raised!")
    return i


def except_as_raise_deletes_target_on_error(x, a):
    """
    >>> except_as_raise_deletes_target_on_error(None, TypeError)
    1
    1
    >>> except_as_raise_deletes_target_on_error(TypeError('test'), TypeError)
    Traceback (most recent call last):
    UnboundLocalError: local variable 'b' referenced before assignment
    >>> except_as_raise_deletes_target_on_error(ValueError('test'), TypeError)
    Traceback (most recent call last):
    ValueError: test
    >>> except_as_raise_deletes_target_on_error(None, TypeError)
    1
    1
    """
    b = 1
    try:
        try:
            i = 1
            if x:
                raise x
        except a as b:
            i = 2
            raise IndexError("TEST")
    except IndexError as e:
        assert 'TEST' in str(e), str(e)
    print(b)  # raises UnboundLocalError if except clause was executed
    return i


def except_as_raise_with_empty_except(x, a):
    """
    >>> except_as_raise_with_empty_except(None, TypeError)
    1
    >>> except_as_raise_with_empty_except(TypeError('test'), TypeError)
    >>> except_as_raise_with_empty_except(ValueError('test'), TypeError)
    Traceback (most recent call last):
    ValueError: test
    >>> except_as_raise_with_empty_except(None, TypeError)
    1
    """
    try:
        if x:
            raise x
        b = 1
    except a as b:  # previously raised UnboundLocalError
        pass
    try:
        print(b)  # raises UnboundLocalError if except clause was executed
    except UnboundLocalError:
        if not x:
            print("unexpected UnboundLocalError raised!")
    else:
        if x:
            print("expected UnboundLocalError not raised!")


def except_as_deletes_target_in_gen(x, a):
    """
    >>> list(except_as_deletes_target_in_gen(None, TypeError))
    [(1, 1), (2, 1), (5, 1)]
    >>> list(except_as_deletes_target_in_gen(TypeError('test'), TypeError))
    [(1, 1), 3, 6]
    >>> list(except_as_deletes_target_in_gen(ValueError('test'), TypeError))
    [(1, 1), (4, 1), (5, 1)]
    """
    b = 1
    try:
        i = 1
        yield (1, b)
        if x:
            raise x
        yield (2, b)
    except a as b:
        i = 2
        assert isinstance(b, a)
        yield 3
    except:
        yield (4, b)
    try:
        yield (5, b)
    except UnboundLocalError:
        yield 6


def nested_except_gh3666(a=False, b=False):
    """
    >>> print(nested_except_gh3666())
    A
    >>> print(nested_except_gh3666(a=True))
    B-V
    >>> print(nested_except_gh3666(a=True, b=True))
    B-V-T
    """
    try:
        if a:
            raise ValueError
        return "A"
    except TypeError as exc:
        return "A-T"
    except ValueError as exc:
        try:
            if b:
                raise TypeError
            return "B-V"
        except ValueError as exc:
            return "B-V-V"
        except TypeError as exc:
            return "B-V-T"


### Py3 feature tests

def print_function(*args):
    """
    >>> print_function(1,2,3)
    1 2 3
    """
    print(*args) # this isn't valid Py2 syntax


def exec3_function(cmd):
    """
    >>> exec3_function('a = 1+1')['a']
    2
    """
    g = {}
    l = {}
    exec(cmd, g, l)
    return l


def exec2_function(cmd):
    """
    >>> exec2_function('a = 1+1')['a']
    2
    """
    g = {}
    exec(cmd, g)
    return g


EXEC_GLOBAL = [5]

def exec1_function(cmd):
    """
    >>> exec1_function('EXEC_GLOBAL.append(1)')
    [1]
    """
    old = len(EXEC_GLOBAL)
    exec(cmd)
    return EXEC_GLOBAL[old:]


ustring = "abcdefg"

def unicode_literals():
    """
    >>> print( unicode_literals() )
    True
    abcdefg
    """
    print(isinstance(ustring, unicode) or type(ustring))
    return ustring


def non_ascii_unprefixed_str():
    """
    >>> s = non_ascii_unprefixed_str()
    >>> isinstance(s, bytes)
    False
    >>> len(s)
    3
    """
    s = 'ø\x20\u0020'
    assert isinstance(s, unicode)
    return s


def non_ascii_raw_str():
    """
    >>> s = non_ascii_raw_str()
    >>> isinstance(s, bytes)
    False
    >>> len(s)
    11
    """
    s = r'ø\x20\u0020'
    assert isinstance(s, unicode)
    return s


def non_ascii_raw_prefixed_unicode():
    """
    >>> s = non_ascii_raw_prefixed_unicode()
    >>> isinstance(s, bytes)
    False
    >>> len(s)
    11
    """
    s = ru'ø\x20\u0020'
    assert isinstance(s, unicode)
    return s


def str_type_is_unicode():
    """
    >>> str_type, s = str_type_is_unicode()
    >>> isinstance(s, type(ustring)) or (s, str_type)
    True
    >>> isinstance(s, str_type) or (s, str_type)
    True
    >>> isinstance(ustring, str_type) or str_type
    True
    """
    cdef str s = 'abc'
    return str, s


def loop_over_unicode_literal():
    """
    >>> print( loop_over_unicode_literal() )
    Py_UCS4
    """
    # Py_UCS4 can represent any Unicode character
    for uchar in 'abcdefg':
        assert uchar in 'abcdefg'
    return cython.typeof(uchar)


def list_comp():
    """
    >>> list_comp()
    [0, 4, 8]
    """
    x = 'abc'
    result = [x*2 for x in range(5) if x % 2 == 0]
    assert x == 'abc' # don't leak in Py3 code
    return result


def list_comp_iterable(it):
    """
    >>> list_comp_iterable([])
    []
    >>> list_comp_iterable([0])
    [0]
    >>> list_comp_iterable([1])
    []
    >>> list_comp_iterable([0, 1])
    [0]
    >>> list_comp_iterable([2])
    [4]
    >>> list_comp_iterable(range(5))
    [0, 4, 8]
    """
    x = 'abc'
    result = [x*2 for x in it if x % 2 == 0]
    assert x == 'abc' # don't leak in Py3 code
    return result


def list_comp_with_lambda():
    """
    >>> list_comp_with_lambda()
    [0, 4, 8]
    """
    x = 'abc'
    result = [x*2 for x in range(5) if (lambda x:x % 2)(x) == 0]
    assert x == 'abc' # don't leak in Py3 code
    return result


class ListCompInClass(object):
    """
    >>> x = ListCompInClass()
    >>> x.listcomp
    [1, 2, 3]
    """
    listcomp = [i+1 for i in range(3)]


cdef class ListCompInCClass:
    """
    >>> x = ListCompInCClass()
    >>> x.listcomp
    [1, 2, 3]
    """
    listcomp = [i+1 for i in range(3)]


module_level_lc = [ module_level_loopvar*2 for module_level_loopvar in range(4) ]
def list_comp_module_level():
    """
    >>> module_level_lc
    [0, 2, 4, 6]
    >>> module_level_loopvar         # doctest: +ELLIPSIS
    Traceback (most recent call last):
    NameError: ...name 'module_level_loopvar' is not defined
    """


module_level_list_genexp = list(module_level_genexp_loopvar*2 for module_level_genexp_loopvar in range(4))
def genexpr_module_level():
    """
    >>> module_level_list_genexp
    [0, 2, 4, 6]
    >>> module_level_genexp_loopvar         # doctest: +ELLIPSIS
    Traceback (most recent call last):
    NameError: ...name 'module_level_genexp_loopvar' is not defined
    """


def list_comp_unknown_type(l):
    """
    >>> list_comp_unknown_type(range(5))
    [0, 4, 8]
    """
    return [x*2 for x in l if x % 2 == 0]


def listcomp_as_condition(sequence):
    """
    >>> listcomp_as_condition(['a', 'b', '+'])
    True
    >>> listcomp_as_condition('ab+')
    True
    >>> listcomp_as_condition('abc')
    False
    """
    if [1 for c in sequence if c in '+-*/<=>!%&|([^~,']:
        return True
    return False


def set_comp():
    """
    >>> sorted(set_comp())
    [0, 4, 8]
    """
    x = 'abc'
    result = {x*2 for x in range(5) if x % 2 == 0}
    assert x == 'abc' # don't leak
    return result


def dict_comp():
    """
    >>> sorted(dict_comp().items())
    [(0, 0), (2, 4), (4, 8)]
    """
    x = 'abc'
    result = {x:x*2 for x in range(5) if x % 2 == 0}
    assert x == 'abc' # don't leak
    return result


# in Python 3, d.keys/values/items() are the iteration methods
@cython.test_assert_path_exists(
    "//WhileStatNode",
    "//WhileStatNode//DictIterationNextNode")
@cython.test_fail_if_path_exists(
    "//ForInStatNode")
def dict_iter(dict d):
    """
    >>> d = {'a' : 1, 'b' : 2, 'c' : 3}
    >>> keys, values, items = dict_iter(d)
    >>> sorted(keys)
    ['a', 'b', 'c']
    >>> sorted(values)
    [1, 2, 3]
    >>> sorted(items)
    [('a', 1), ('b', 2), ('c', 3)]

    >>> dict_iter({})
    ([], [], [])
    """
    keys = [ key for key in d.keys() ]
    values = [ value for value in d.values() ]
    items = [ item for item in d.items() ]
    return keys, values, items


@cython.test_assert_path_exists(
    "//WhileStatNode",
    "//WhileStatNode//DictIterationNextNode")
@cython.test_fail_if_path_exists(
    "//ForInStatNode")
def dict_iter_new_dict():
    """
    >>> dict_keys, keys, values, items = dict_iter_new_dict()
    >>> sorted(dict_keys)
    [11, 22, 33]
    >>> sorted(keys)
    [11, 22, 33]
    >>> sorted(values)
    [1, 2, 3]
    >>> sorted(items)
    [(11, 1), (22, 2), (33, 3)]
    """
    dict_keys = [ key for key in {11 : 1, 22 : 2, 33 : 3} ]
    keys = [ key for key in {11 : 1, 22 : 2, 33 : 3}.keys() ]
    values = [ value for value in {11 : 1, 22 : 2, 33 : 3}.values() ]
    items = [ item for item in {11 : 1, 22 : 2, 33 : 3}.items() ]
    return dict_keys, keys, values, items


def int_literals():
    """
    >>> int_literals()
    long
    long
    unsigned long
    unsigned long
    """
    print(cython.typeof(1L))
    print(cython.typeof(10000000000000L))
    print(cython.typeof(1UL))
    print(cython.typeof(10000000000000UL))


def annotation_syntax(a: "test new test", b : "other" = 2, *args: "ARGS", **kwargs: "KWARGS") -> "ret":
    """
    >>> annotation_syntax(1)
    3
    >>> annotation_syntax(1,3)
    4

    >>> len(annotation_syntax.__annotations__)
    5
    >>> print(annotation_syntax.__annotations__['a'])
    'test new test'
    >>> print(annotation_syntax.__annotations__['b'])
    'other'
    >>> print(annotation_syntax.__annotations__['args'])
    'ARGS'
    >>> print(annotation_syntax.__annotations__['kwargs'])
    'KWARGS'
    >>> print(annotation_syntax.__annotations__['return'])
    'ret'
    """
    result : int = a + b

    return result


@cython.annotation_typing(False)
def builtin_as_ignored_annotation(text: str):
    # Used to crash the compiler when annotation typing is disabled.
    # See https://github.com/cython/cython/issues/2811
    """
    >>> builtin_as_ignored_annotation("abc")
    a
    b
    c
    """
    for c in text:
        print(c)


@cython.annotation_typing(True)
def int_annotation(x: int) -> int:
    """
    >>> print(int_annotation(1))
    2
    >>> print(int_annotation(10))
    1024
    >>> print(int_annotation(100))
    1267650600228229401496703205376
    >>> print(int_annotation((10 * 1000 * 1000) // 1000 // 1000))  # 'long' arg in Py2
    1024
    >>> print(int_annotation((100 * 1000 * 1000) // 1000 // 1000))  # 'long' arg in Py2
    1267650600228229401496703205376
    """
    return 2 ** x


@cython.annotation_typing(True)
async def async_def_annotations(x: 'int') -> 'float':
    """
    >>> ret, arg = sorted(async_def_annotations.__annotations__.items())
    >>> print(ret[0]); print(ret[1])
    return
    'float'
    >>> print(arg[0]); print(arg[1])
    x
    'int'
    """
    return float(x)


def const_str_index(int n):
    """
    >>> const_str_index(0)
    '0'
    >>> const_str_index(12)
    '1'
    """
    return str(n)[0]


@cython.test_fail_if_path_exists(
    "//CoerceToPyTypeNode",
)
@cython.test_assert_path_exists(
    "//MulNode[@is_sequence_mul = True]",
)
def string_multiply(str s, int N):
    """
    >>> print(string_multiply(u"abc", 3))
    abcabcabc
    """
    return s * N


@cython.test_fail_if_path_exists(
    "//CoerceToPyTypeNode",
)
@cython.test_assert_path_exists(
    "//MulNode[@is_sequence_mul = True]",
)
def string_multiply_call(s, int N):
    """
    >>> print(string_multiply_call(u"abc", 3))
    abcabcabc
    """
    return str(s) * N



# mode: run


def test(k):
    """
    >>> test(0)
    0
    >>> test(1)
    1
    >>> test(256 ** unsigned_long_size() - 1) > 0
    True
    """
    cdef unsigned long m
    m = k
    return m


def unsigned_long_size():
    return sizeof(unsigned long)



from cython.parallel import parallel, prange
import cython
from cython.cimports.libc.math import sqrt

@cython.boundscheck(False)
@cython.wraparound(False)
def normalize(x: cython.double[:]):
    i: cython.Py_ssize_t
    total: cython.double = 0
    norm: cython.double
    with cython.nogil, parallel():
        for i in prange(x.shape[0]):
            total += x[i]*x[i]
        norm = sqrt(total)
        for i in prange(x.shape[0]):
            x[i] /= norm



__doc__ = u"""
    >>> class Test(object):
    ...     def __init__(self, i):
    ...         self.i = i
    >>> b = Test(1)
    >>> b.spam = Test(2)
    >>> b.spam.eggs = Test(3)
    >>> b.spam.eggs.spam = Test(4)
    >>> b.spam.eggs.spam.eggs = Test(5)

    >>> a = f(b)
    >>> a.i
    2
    >>> b.i
    1
    >>> a.spam.i
    1
    >>> b.spam.i
    2
    >>> a.spam.eggs.i
    Traceback (most recent call last):
    AttributeError: 'Test' object has no attribute 'eggs'
    >>> b.spam.eggs.i
    3
    >>> a.spam.spam.i
    2
    >>> b.spam.spam.i
    1
    >>> a.spam.eggs.spam.i
    Traceback (most recent call last):
    AttributeError: 'Test' object has no attribute 'eggs'
    >>> b.spam.eggs.spam.i
    4

    >>> a = g(b)
    >>> a.i
    3
    >>> b.i
    1
    >>> a.spam.i
    4
    >>> b.spam.i
    2
    >>> a.spam.eggs.i
    1
    >>> b.spam.eggs.i
    3
    >>> a.spam.spam.i
    Traceback (most recent call last):
    AttributeError: 'Test' object has no attribute 'spam'
    >>> b.spam.spam.i
    1
    >>> a.spam.eggs.spam.i
    2
    >>> b.spam.eggs.spam.i
    4
"""

def f(b):
    a = b.spam
    a.spam = b
    return a

def g(b):
    a = b.spam.eggs
    a.spam.eggs = b
    return a



import cython

@cython.cclass
class Shrubbery:
    width = cython.declare(cython.int, visibility='public')
    height = cython.declare(cython.int, visibility='public')
    depth = cython.declare(cython.float, visibility='readonly')



# mode: run

def f():
    """
    >>> f()
    (1, 2, 1, 2)
    """
    cdef object obj1a, obj2a, obj3a, obj1b, obj2b, obj3b
    obj1b, obj2b, obj3b = 1, 2, 3
    obj1a, obj2a = obj1b, obj2b
    return obj1a, obj2a, obj1b, obj2b

def g():
    """
    >>> g()
    (1, 1, 2, 2, 3, 3)
    """
    cdef object obj1a, obj2a, obj3a, obj1b, obj2b, obj3b
    obj1b, obj2b, obj3b = 1, 2, 3
    obj1a, [obj2a, obj3a] = [obj1b, (obj2b, obj3b)]
    return obj1a, obj1b, obj2a, obj2b, obj3a, obj3b

def h():
    """
    >>> h()
    (1, b'test', 3, 1, b'test', 3)
    """
    cdef object obj1a, obj2a, obj3a, obj1b, obj2b, obj3b
    cdef int int1, int2
    cdef char *ptr1, *ptr2
    int2, ptr2, obj1b = 1, "test", 3
    int1, ptr1, obj1a = int2, ptr2, obj1b
    return int1, ptr1, obj1a, int2, ptr2, obj1b

def j():
    """
    >>> j()
    (2, 1, 4, 2, 6, 3)
    """
    cdef object obj1a, obj2a, obj3a, obj1b, obj2b, obj3b
    obj1b, obj2b, obj3b = 1, 2, 3
    obj1a, obj2a, obj3a = obj1b + 1, obj2b + 2, obj3b + 3
    return obj1a, obj1b, obj2a, obj2b, obj3a, obj3b



import cython

@cython.annotation_typing(False)
def function_without_typing(a: int, b: int) -> int:
    """Cython is ignoring annotations in this function"""
    c: int = a + b
    return c * a


@cython.annotation_typing(False)
@cython.cclass
class NotAnnotatedClass:
    """Cython is ignoring annotatons in this class except annotated_method"""
    d: dict

    def __init__(self, dictionary: dict):
        self.d = dictionary

    @cython.annotation_typing(True)
    def annotated_method(self, key: str, a: cython.int, b: cython.int):
        prefixed_key: str = 'prefix_' + key
        self.d[prefixed_key] = a + b


def annotated_function(a: cython.int, b: cython.int):
    s: cython.int = a + b
    with cython.annotation_typing(False):
        # Cython is ignoring annotations within this code block
        c: list = []
    c.append(a)
    c.append(b)
    c.append(s)
    return c



from cython.cimports.sin_of_square import Function

@cython.cclass
class WaveFunction(Function):

    # Not available in Python-space:
    offset: float

    # Available in Python-space:
    freq = cython.declare(cython.double, visibility='public')

    # Available in Python-space, but only for reading:
    scale = cython.declare(cython.double, visibility='readonly')

    # Available in Python-space:
    @property
    def period(self):
        return 1.0 / self.freq

    @period.setter
    def period(self, value):
        self.freq = 1.0 / value



# mode: error

cdef void foo():
	cdef int bool, int1
	cdef char *ptr2
	cdef int *ptr3
	cdef object i = 5

	bool = i == ptr2  # evaluated in Python space
	bool = ptr3 == i # error
	bool = int1 == ptr2 # error
	bool = ptr2 == ptr3 # error

	bool = 1 in 2 in 3

_ERRORS = u"""
10:13: Invalid types for '==' (int *, Python object)
11:13: Invalid types for '==' (int, char *)
12:13: Invalid types for '==' (char *, int *)
"""



# mode: compile

cdef void foo():
    cdef int i1, i2=0
    cdef char c1=0, c2
    cdef char *p1, *p2=NULL
    cdef object obj1
    i1 = i2
    i1 = c1
    p1 = p2
    obj1 = i1
    i1 = obj1
    p1 = obj1
    p1 = "spanish inquisition"

foo()



# mode: run
# tag: pure3.0
# cython: binding=True

"""
Test that fused functions can be used in the same way as CyFunctions with respect to
assigning them to class attributes. Previously they enforced extra type/argument checks
beyond those which CyFunctions did.
"""

import cython

MyFusedClass = cython.fused_type(
    float,
    'Cdef',
    object)

def fused_func(x: MyFusedClass):
    return (type(x).__name__, cython.typeof(x))

IntOrFloat = cython.fused_type(int, float)

def fused_func_0(x: IntOrFloat = 0):
    """
    Fused functions can legitimately take 0 arguments
    >>> fused_func_0()
    ('int', 'int')

    # subscripted in module __doc__ conditionally
    """
    return (type(x).__name__, cython.typeof(x))

def regular_func(x):
    return (type(x).__name__, cython.typeof(x))

def regular_func_0():
    return

@classmethod
def fused_classmethod_free(cls, x: IntOrFloat):
    return (cls.__name__, type(x).__name__)

@cython.cclass
class Cdef:
    __doc__ = """
    >>> c = Cdef()

    # functions are callable with an instance of c
    >>> c.fused_func()
    ('Cdef', 'Cdef')
    >>> c.regular_func()
    ('Cdef', '{typeofCdef}')
    >>> c.fused_in_class(1.5)
    ('float', 'float')

    # Fused functions are callable without an instance
    # (This applies to everything in Py3 - see __doc__ below)
    >>> Cdef.fused_func(1.5)
    ('float', 'float')
    >>> Cdef.fused_in_class(c, 1.5)
    ('float', 'float')
    >>> Cdef.fused_func_0()
    ('int', 'int')

    # Functions not expecting an argument don't work with an instance
    >>> c.regular_func_0()  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: regular_func_0() takes ... arguments ...1... given...

    # Looking up a class attribute doesn't go through all of __get__
    >>> Cdef.fused_in_class is Cdef.fused_in_class
    True

    # looking up a classmethod does go through __get__ though
    >>> Cdef.fused_classmethod is Cdef.fused_classmethod
    False
    >>> Cdef.fused_classmethod_free is Cdef.fused_classmethod_free
    False
    >>> Cdef.fused_classmethod(1)
    ('Cdef', 'int')
    >>> Cdef.fused_classmethod_free(1)
    ('Cdef', 'int')
    """.format(typeofCdef = 'Python object' if cython.compiled else 'Cdef')

    if cython.compiled:
        __doc__ += """

    # fused_func_0 does not accept a "Cdef" instance
    >>> c.fused_func_0()
    Traceback (most recent call last):
    TypeError: No matching signature found

    # subscripting requires fused methods (so  not pure Python)
    >>> Cdef.fused_func_0['float']()
    ('float', 'float')
    >>> c.fused_func_0['float']()  # doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    TypeError: (Exception looks quite different in Python2 and 3 so no way to match both)

    >>> Cdef.fused_classmethod['float'] is Cdef.fused_classmethod['float']
    False
    >>> Cdef.fused_classmethod_free['float'] is Cdef.fused_classmethod_free['float']
    False
    """
    fused_func = fused_func
    fused_func_0 = fused_func_0
    regular_func = regular_func
    regular_func_0 = regular_func_0

    fused_classmethod_free = fused_classmethod_free

    def fused_in_class(self, x: MyFusedClass):
        return (type(x).__name__, cython.typeof(x))

    def regular_in_class(self):
        return type(self).__name__

    @classmethod
    def fused_classmethod(cls, x: IntOrFloat):
        return (cls.__name__, type(x).__name__)

class Regular(object):
    __doc__ = """
    >>> c = Regular()

    # Functions are callable with an instance of C
    >>> c.fused_func()
    ('Regular', '{typeofRegular}')
    >>> c.regular_func()
    ('Regular', '{typeofRegular}')

    # Fused functions are callable without an instance
    # (This applies to everything in Py3 - see __doc__ below)
    >>> Regular.fused_func(1.5)
    ('float', 'float')
    >>> Regular.fused_func_0()
    ('int', 'int')

    # Functions not expecting an argument don't work with an instance
    >>> c.regular_func_0()  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: regular_func_0() takes ... arguments ...1... given...

    # Looking up a class attribute doesn't go through all of __get__
    >>> Regular.fused_func is Regular.fused_func
    True

    # looking up a classmethod does go __get__ though
    >>> Regular.fused_classmethod is Regular.fused_classmethod
    False
    >>> Regular.fused_classmethod_free is Regular.fused_classmethod_free
    False
    >>> Regular.fused_classmethod(1)
    ('Regular', 'int')
    >>> Regular.fused_classmethod_free(1)
    ('Regular', 'int')
    """.format(typeofRegular = "Python object" if cython.compiled else 'Regular')
    if cython.compiled:
        __doc__ += """
    # fused_func_0 does not accept a "Regular" instance
    >>> c.fused_func_0()
    Traceback (most recent call last):
    TypeError: No matching signature found

    # subscripting requires fused methods (so  not pure Python)
    >>> c.fused_func_0['float']()  # doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    TypeError: (Exception looks quite different in Python2 and 3 so no way to match both)
    >>> Regular.fused_func_0['float']()
    ('float', 'float')

    >>> Regular.fused_classmethod['float'] is Regular.fused_classmethod['float']
    False
    >>> Regular.fused_classmethod_free['float'] is Regular.fused_classmethod_free['float']
    False
    """

    fused_func = fused_func
    fused_func_0 = fused_func_0
    regular_func = regular_func
    regular_func_0 = regular_func_0

    fused_classmethod_free = fused_classmethod_free

    @classmethod
    def fused_classmethod(cls, x: IntOrFloat):
        return (cls.__name__, type(x).__name__)


# extra Py3 only tests - shows that functions added to a class can be called
# with an type as the first argument
__doc__ = """
    >>> Cdef.regular_func(1.5)
    ('float', '{typeoffloat}')
    >>> Regular.regular_func(1.5)
    ('float', '{typeoffloat}')
    >>> Cdef.regular_func_0()
    >>> Regular.regular_func_0()
""".format(typeoffloat='Python object' if cython.compiled else 'float')

if cython.compiled:
    __doc__ += """
    >>> fused_func_0['float']()
    ('float', 'float')
    """



# mode: error

ctypedef struct struct_type_not_boolean:
    int i
    float f

if struct_type_not_boolean:
    print("INVALID CODE")

cdef struct struct_not_boolean:
    int i
    float f

if struct_not_boolean:
    print("INVALID CODE")

ctypedef union union_type_not_boolean:
    int i
    float f

if union_type_not_boolean:
    print("INVALID CODE")

cdef union union_not_boolean:
    int i
    float f

if union_not_boolean:
    print("INVALID CODE")


_ERRORS = u"""
7:3: 'struct_type_not_boolean' is not a constant, variable or function identifier
7:3: Type 'struct_type_not_boolean' not acceptable as a boolean

14:3: 'struct_not_boolean' is not a constant, variable or function identifier
14:3: Type 'struct_not_boolean' not acceptable as a boolean

21:3: 'union_type_not_boolean' is not a constant, variable or function identifier
21:3: Type 'union_type_not_boolean' not acceptable as a boolean

28:3: 'union_not_boolean' is not a constant, variable or function identifier
28:3: Type 'union_not_boolean' not acceptable as a boolean
"""



# cython: language_level=3

#
#  Example of an extension type.
#

cdef class Spam:
    cdef public int amount

    def __cinit__(self):
        self.amount = 0

    def __dealloc__(self):
        print(self.amount, "tons of spam is history.")

    def get_amount(self):
        return self.amount

    def set_amount(self, new_amount):
        self.amount = new_amount

    def describe(self):
        print(self.amount, "tons of spam!")




cdef class Owner:
    cdef object x

cdef call_me_with_owner(Owner owner, x):
    owner.x = "def" # overwrite external reference
    return x        # crashes if x is not owned by function or caller

def test_ext_type_attr():
    """
    >>> test_ext_type_attr()
    'abc5'
    """
    owner = Owner()
    owner.x = ''.join("abc%d" % 5) # non-interned object
    return call_me_with_owner(owner, owner.x)


cdef void call_me_without_gil(Owner owner, x) with gil:
    owner.x = "def" # overwrite external reference
    print x         # crashes if x is not owned by function or caller

def test_ext_type_attr_nogil():
    """
    >>> test_ext_type_attr_nogil()
    abc5
    """
    owner = Owner()
    owner.x = ''.join("abc%d" % 5) # non-interned object
    with nogil:
        call_me_without_gil(owner, owner.x)


# the following isn't dangerous as long as index access uses temps

cdef call_me_with_list(list l, x):
    l[:] = [(1,2), (3,4)] # overwrite external reference
    return x              # crashes if x is not owned by function or caller

def test_index():
    """
    >>> test_index()
    [3, 4]
    """
    l = [[1,2],[3,4]]
    return call_me_with_list(l, l[1])



# cython: binding=True
# mode: run
# tag: cyfunction, cpp

from libcpp.vector cimport vector

cdef class A:
    def f1(self, a, b=1, vector[double] c = vector[double]()):
        pass
    def f2(self, a, b=1,/, vector[double] c = vector[double](1, 2.0)):
        pass
    def f3(self, a, /, b=1, *, c = vector[double](2, 3.0)):
        pass


def check_defaults_on_methods():
    """
    >>> A.f1.__defaults__
    (1, [])
    >>> A.f1.__kwdefaults__
    >>> A.f2.__defaults__
    (1, [2.0])
    >>> A.f2.__kwdefaults__
    >>> A.f3.__defaults__
    (1,)
    >>> A.f3.__kwdefaults__
    {'c': [3.0, 3.0]}
    """
    pass



# mode: error

cdef struct Foo

ctypedef struct Foo:
	int i

ctypedef struct Blarg:
	char c

cdef struct Blarg

cdef Foo f
cdef Blarg b


_ERRORS = u"""
5:0: 'Foo' previously declared using 'cdef'
11:0: 'Blarg' previously declared using 'ctypedef'
"""



# mode: run
# tag: cpp, cpp11

# cython: language_level=3

from libcpp.set cimport multiset
from libcpp.unordered_set cimport unordered_multiset

def test_multiset_insert(vals):
    """
    >>> test_multiset_insert([1,2,2,3, -1])
    [-1, 1, 2, 2, 3]
    """
    cdef multiset[int] ms = multiset[int]()
    cdef multiset[int].iterator it
    for v in vals:
        it = ms.insert(v)
    return [ item for item in ms ]

def test_multiset_insert_it(vals):
    """
    >>> test_multiset_insert_it([1,2,2,3, -1])
    [-1, 1, 2, 2, 3]
    """
    cdef unordered_multiset[int] ums = unordered_multiset[int]()
    cdef multiset[int] ms = multiset[int]()
    for v in vals:
        ums.insert(v)
    ms.insert(ums.begin(), ums.end())
    return [ item for item in ms ]

def test_const_multiset_insert_it(vals):
    """
    >>> test_const_multiset_insert_it([1,2,2,3, -1])
    [-1, 1, 2, 2, 3]
    """
    cdef unordered_multiset[int] ums = unordered_multiset[int]()
    cdef multiset[int] ms = multiset[int]()
    for v in vals:
        ums.insert(v)
    ms.insert(ums.cbegin(), ums.cend())
    return [ item for item in ms ]

def test_multiset_count(vals, to_find):
    """
    >>> test_multiset_count([1,2,2,3, -1], 1)
    1
    >>> test_multiset_count([1,2,2,3, -1], 2)
    2
    """
    cdef multiset[int] ms = multiset[int]()
    for v in vals:
        ms.insert(v)
    return ms.count(to_find)

def test_multiset_erase(vals, int to_remove):
    """
    >>> test_multiset_erase([1,2,2,3, -1], 1)
    [-1, 2, 2, 3]
    >>> test_multiset_erase([1,2,2,3, -1], 2)  # removes both copies of 2
    [-1, 1, 3]
    """
    cdef multiset[int] ms = multiset[int]()
    cdef size_t ret
    for v in vals:
        ms.insert(v)
    ret = ms.erase(to_remove)
    return [ item for item in ms ]

def test_multiset_find_erase(vals, to_remove):
    """
    >>> test_multiset_find_erase([1,2,2,3, -1], 1)
    [-1, 2, 2, 3]
    >>> test_multiset_find_erase([1,2,2,3, -1], 2)  # removes a single copy of 2
    [-1, 1, 2, 3]
    """
    cdef multiset[int] ms = multiset[int]()
    cdef multiset[int].iterator it
    for v in vals:
        ms.insert(v)
    it = ms.find(to_remove)
    it = ms.erase(it)
    return [ item for item in ms ]


def test_unordered_multiset_insert(vals):
    """
    >>> test_unordered_multiset_insert([1,2,2,3, -1])
    [-1, 1, 2, 2, 3]
    """
    cdef unordered_multiset[int] ums = unordered_multiset[int]()
    cdef unordered_multiset[int].iterator it
    for v in vals:
        it = ums.insert(v)
    return sorted([ item for item in ums ])

def test_unordered_multiset_insert_it(vals):
    """
    >>> test_unordered_multiset_insert_it([1,2,2,3, -1])
    [-1, 1, 2, 2, 3]
    """
    cdef multiset[int] ms = multiset[int]()
    cdef unordered_multiset[int] ums = unordered_multiset[int]()
    for v in vals:
        ms.insert(v)
    ums.insert(ms.begin(), ms.end())
    return sorted([ item for item in ums ])

def test_const_unordered_multiset_insert_it(vals):
    """
    >>> test_const_unordered_multiset_insert_it([1,2,2,3, -1])
    [-1, 1, 2, 2, 3]
    """
    cdef multiset[int] ms = multiset[int]()
    cdef unordered_multiset[int] ums = unordered_multiset[int]()
    for v in vals:
        ms.insert(v)
    ums.insert(ms.cbegin(), ms.cend())
    return sorted([ item for item in ums ])

def test_unordered_multiset_count(vals, to_find):
    """
    >>> test_unordered_multiset_count([1,2,2,3, -1], 1)
    1
    >>> test_unordered_multiset_count([1,2,2,3, -1], 2)
    2
    """
    cdef unordered_multiset[int] ums = unordered_multiset[int]()
    for v in vals:
        ums.insert(v)
    return ums.count(to_find)

def test_unordered_multiset_erase(vals, int to_remove):
    """
    >>> test_unordered_multiset_erase([1,2,2,3, -1], 1)
    [-1, 2, 2, 3]
    >>> test_unordered_multiset_erase([1,2,2,3, -1], 2)  # removes both copies of 2
    [-1, 1, 3]
    """
    cdef unordered_multiset[int] ums = unordered_multiset[int]()
    cdef size_t ret
    for v in vals:
        ums.insert(v)
    ret = ums.erase(to_remove)
    return sorted([ item for item in ums ])

def test_unordered_multiset_find_erase(vals, to_remove):
    """
    >>> test_unordered_multiset_find_erase([1,2,2,3, -1], 1)
    [-1, 2, 2, 3]
    >>> test_unordered_multiset_find_erase([1,2,2,3, -1], 2)  # removes a single copy of 2
    [-1, 1, 2, 3]
    """
    cdef unordered_multiset[int] ums = unordered_multiset[int]()
    cdef unordered_multiset[int].iterator it
    for v in vals:
        ums.insert(v)
    it = ums.find(to_remove)
    it = ums.erase(it)
    return sorted([ item for item in ums ])


def test_unordered_multiset_misc():
    """
    >>> test_unordered_multiset_misc()
    """
    cdef unordered_multiset[int] ms = unordered_multiset[int]()
    ms.insert(1)
    assert ms.load_factor() > 0
